
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_cioslowski
  use f90_kind

  private 
  public :: create_cioslowski,print_cioslowski

  !************************************************************************

contains

  !************************************************************************

  subroutine create_cioslowski(dipol_dxyz,cioslowski)
    !----------------------------------------------------------------
    ! written by Markus Reiher and Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(in) :: dipol_dxyz(:,:)
    real(double),pointer :: cioslowski(:)
    integer natoms,atom
    natoms=ubound(dipol_dxyz,2)/3
    allocate(cioslowski(natoms))
    do atom=1,natoms
       cioslowski(atom) = (dipol_dxyz(1,3*atom-2) &
            &           + dipol_dxyz(2,3*atom-1) &
            &           + dipol_dxyz(3,3*atom))/3.d0

    enddo
  end subroutine create_cioslowski

  !************************************************************************

  subroutine print_cioslowski(out,cioslowski)
    !----------------------------------------------------------------
    ! written by Markus Reiher and Carsten Kind, Uni Erlangen, 2000
    ! slightly modified by markus reiher, aug. 2001
    !----------------------------------------------------------------
    implicit none
    integer ,intent(in) :: out
    real(double),intent(in) :: cioslowski(:)
    integer natoms,atom
    write(out,*)
    write(out,'('' Cioslowski charge decomposition'')')
    write(out,'('' ------------------------------- '')')
    write (out,'('' Charges derived from generalized atomic polar tensors'')') 
    write (out,'('' according to J. Cioslowski, JACS 111 (1989) 8333-8336.'')') 
    write(out,*)
    write(out,'('' | atom | gapt charge |'')')
    write(out,'('' |------|-------------|'')')
    natoms=ubound(cioslowski,1)
    do atom=1,natoms
       write(out,'('' |'',1X,i4,'' |'',f12.4,'' |'')') atom,cioslowski(atom)
    enddo
    write(out,*)
    write(out,'(''   total gapt charge:'',f12.4)')sum(cioslowski(:))
  end subroutine print_cioslowski

  !************************************************************************

end module snf_cioslowski


