
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
program snfdefine
  !----------------------------------------------------------------
  ! this program modifies the control file to prepare a correct 
  ! dscf/ridft/grad/rdgrad/moloch/escf - run
  ! and writes an restart file, such that only irreducible atoms 
  ! will be calculated. A define - run might be necessary after
  ! snf_define.
  !
  ! written by Carsten Kind, Uni Erlangen, 2001
  ! modified and extended by Johannes Neugebauer, Uni Erlangen, 2001
  ! modified by Markus reiher, Uni Erlangen, Nov. 2001
  !----------------------------------------------------------------
  use f90_kind
  use snf_molecule,       only: type_atom,print_molecule,destroy_molecule,&
       &                        fiso_mass,find_mass,print_isomass,&
       &                        isotope_equal,atom_equal,setisomass,testlin
  use snf_char_table,     only: type_char_table,create_char_table,&
       &                        destroy_char_table,print_char_table,checkpg
  use snf_restart,        only: write_restart,prepare_dummy_restart,&
       &                        backup_restart,check_restart_status,read_restart
  use snf_control,        only: type_ctrl,read_control,print_ctrl,init_ctrl,&
       &                        write_control, copy_tm_to_ctrl,copy_ctrl_to_tm,&
       &                        destroy_control,getdfreq,get_energy,&
       &                        get_energy_mp2,copy_ctrl_to_dal,get_spin
!MAXSYM>
  use snf_irred_dist,     only: create_irred_dist,prepare_restart2
!MAXSYM<
  use snf_irred_atoms,    only: create_irred_atoms,destroy_irred_atoms, &
       &                        print_irred_atoms,check_geometry,get_point_grp,&
       &                        is_point_grp
  use snf_dalton,         only: init_dal_ctrl,write_dal_ctrl
  use snf_turbomole,      only: read_tm_coord,read_tm_ctrl,modify_tm_ctrl,&
       &                        init_tm_ctrl
  use snf_tools,          only: string2integer,string2real,clean,prog_type,&
       &                        testfile,bfile
  use snf_error,          only: serror
  use snf_param
  implicit none

  type(type_ctrl)          :: &
       & ctrl,ctrl_restart     ! essential control flags and parameters
  type(type_atom), pointer :: &
       & molecule(:)           ! geometry atoms and atomic masses of the molecule  
  type(type_char_table)    :: &
       & char_table            ! character table
  integer,pointer          :: &
       &  istepfl(:), &        ! points to calculate(=6*nirred_atoms)
       &  oistepfl(:), &       ! 
       & atom_index(:), &      ! index of irreducible atom (1:natoms)
       & symm_index(:), &      ! symmetry op. to optain irred. atom (1:natoms)
       & irred_atom_index(:),& ! index list with irreduzible atoms (1:nirred_atoms)
       & selatom(:)            ! atom-numbers which special isotopes are selected for
!MAXSYM>
  real(double),allocatable :: &
       & distortion(:)         ! generalized distortion in the cartesian basis
  integer,pointer          :: &
       & dist_index(:), &    ! index of irreducible dist.
       & dsymm_index(:), &   ! symmetry op. to optain irred. dist.
       & irred_dist_index(:) ! index list with irreducible dist.
  integer :: nirred_dist,orinum
!MAXSYM<
  integer iiat,iidum,iixyz,iidiraus,iidir
  integer jiat,jidum,jixyz,jidiraus,jidir
  integer :: nirred_atoms,i,status,pgstat,nselatom,ipre,ipreo,j
  real(double),pointer :: gradients(:,:,:),dipols(:,:,:),pols(:,:,:,:),massatom(:)
  real(double),pointer :: ngradients(:,:,:),ndipols(:,:,:),npols(:,:,:,:)
  real(double) :: cthres       ! lowest acceptable value for cstep
  character(len=8),pointer :: &
       & nvmach(:),           & ! list of all used machines 
       & onvmach(:)             ! 
  logical :: valid,found_control,fconhere,wrestart,goback,crap
  character(len=132) :: zeile,errbuff
  character(len=80) :: jobfile
  character(len=3) :: pointgrp
  character :: c
  parameter (cthres=1.0d-3)
  !----------------------------------------------------------------
  ! print header
  !----------------------------------------------------------------
  call pr_snfdefhead
  !----------------------------------------------------------------
  ! initialize variables
  !----------------------------------------------------------------
  nullify(gradients,dipols,pols,istepfl,molecule, &
       &  atom_index,symm_index,irred_atom_index,massatom,selatom)
  nullify(ngradients,ndipols,npols,oistepfl,onvmach)
  call testfile('control')
  call testfile('coord')
  call init_ctrl(ctrl)
  found_control=.false.
  crap=.false.
  !----------------------------------------------------------------
  ! read file snf_control, if this exists 
  !----------------------------------------------------------------
  inquire(file='snf_control',exist=fconhere)
  if(fconhere) call old_snfcont(ctrl,selatom,massatom,molecule,c,pointgrp)
  if(c.eq.'n') goto 200
  !----------------------------------------------------------------
  ! start interactive part
  !----------------------------------------------------------------
110 call alt_cont(ctrl%control,found_control)
  !----------------------------------------------------------------
  ! read default data if ctrl%control is present.
  ! if .not.found_control, but ./snf_control exists,
  ! then read default data from ./snf_control.
  ! coordinates are  read in next block !
  !----------------------------------------------------------------
  if(found_control) then
     call read_control(ctrl,selatom,massatom)
  else if(fconhere) then
     ctrl%control='snf_control'
     call read_control(ctrl,selatom,massatom)
  endif
  orinum=ctrl%numderiv
  !----------------------------------------------------------------
  ! start interactive input:
  ! select program for single point calculations
  !----------------------------------------------------------------
120 if(associated(molecule)) deallocate(molecule)
  call selprog(ctrl,molecule,goback)
  if(goback) goto 110
130 if(associated(ctrl%dal_ctrl)) then
    call dalopt(ctrl,goback)
    if(goback) goto 120
  end if
  !----------------------------------------------------------------
  ! set masses for isotopes found in snf_control
  !----------------------------------------------------------------
  molecule(:)%changem=.false.
  if(associated(selatom)) call setisomass(molecule,selatom,massatom)
  !----------------------------------------------------------------
  ! test linearity of molecule
  !----------------------------------------------------------------
  if(testlin(molecule)) then
    if(ctrl%symmetry.eq.'c6v') then
      call serror('snf_define',-1,'Found linear molecule. Symmetry ##&
       & is set to C4v to use the full symmetry.')
      ctrl%symmetry='c4v'
    else if(ctrl%symmetry.eq.'d6h') then
      call serror('snf_define',-1,'Found linear molecule. Symmetry ##&
       & is set to D4h to use the full symmetry.')
      ctrl%symmetry='d4h'
    else
      call serror('snf_define',-1,'Found linear molecule. Set symmetry ##&
      & to C4v or D4h, resp., to use the full symmetry.')
    end if
  end if
  !----------------------------------------------------------------
  ! check point-group
  !----------------------------------------------------------------
  pgstat=-1
  call checkpg(ctrl%symmetry,pgstat)
  if (pgstat.lt.1) then
    if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
         ctrl%symmetry,'" is not implemented'
    if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
         ctrl%symmetry
    call serror('snf_define',2,errbuff)
  end if
  if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%symmetry)) then
     call get_point_grp(molecule,1.D-3,atom_equal,pointgrp)
     write(*,*)
     write(*,*)'Point-group ',Ctrl%symmetry,' does not fit to coordinates.'
     write(*,*)'Automatic point-group check yields: ',pointgrp
     Ctrl%symmetry=pointgrp
  endif
  pgstat=-1
  if(ctrl%isosym .ne. 'def') then
     call checkpg(ctrl%isosym,pgstat)
     if (pgstat.lt.1) then
       if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
            ctrl%symmetry,'" is not implemented'
       if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
            ctrl%symmetry
        call serror('snf_define',2,errbuff)
     end if
     if(.not.is_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)) then
        call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
        write(*,*)
        write(*,*)'Point-group ',Ctrl%isosym,' does not fit to coordinates'
        write(*,*)'if different isotopes are taken into account.'
        write(*,*)'Automatic point-group check yields: ',pointgrp
        Ctrl%isosym='def'
     endif
  endif
  !----------------------------------------------------------------
  ! small output to validate right inputfile
  !----------------------------------------------------------------
  write(*,*)
  write(*,'(''  Symmetry     : '',a3)')ctrl%symmetry
  if(ctrl%isosym.ne.'def') then
     write(*,'(''  Symmetry with selected isotopes    : '',a3)')ctrl%isosym
  end if
  write(*,'(''  No. of atoms : '',i3)')ubound(molecule,1)
  write(*,*)
  call print_molecule(molecule)

140 valid=.false.
    do while(.not.valid)
     write(*,*)
     write(*,*)
     write(*,*) ' If you want to enter one of the following menus'
     write(*,*) ' then type the corresponding command.'
     write(*,*)
     write(*,*) '   iso      : isotope menu'
     write(*,*) '   num      : numerical derivative menu'
     write(*,*) '   spe      : spectrum settings menu'
     write(*,*) '   the      : thermodynamics menu'
     write(*,*) '   &        : go back to program selection'
     write(*,*) '   >return< : quit this menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(len_trim(zeile).eq.0) then
       valid=.true.
     else if(len_trim(zeile).eq.1.and.zeile(1:1).eq.'&') then
       if(associated(ctrl%dal_ctrl)) then
         goto 130
       else
         goto 120
       end if
     else if(len_trim(zeile).eq.3) then
       select case(zeile(1:3))
        case ('iso')
          call isomenu(ctrl,molecule,selatom,massatom,pointgrp,pgstat)
        case ('num')
          call nummenu(ctrl)
        case ('spe')
          call specmenu(ctrl)
        case ('the')
          call thermmenu(ctrl)
        case default
       end select
     end if
   end do
  !----------------------------------------------------------------
  ! Raman intensities menu
  !----------------------------------------------------------------
160 if(ctrl%mode.ne.3.and.ctrl%mode.ne.4.and.ctrl%mode.ne.21) then
      call ramanmenu(ctrl,goback)
    else
      write(*,*) ' Skipping Raman menu since Raman calculations'
      write(*,*) ' cannot be performed in this program mode'
    end if
  if(goback) goto 140
  !----------------------------------------------------------------
  ! Output options menu
  !----------------------------------------------------------------
170 call outmenu(ctrl,goback)
   if((ctrl%mode.eq.3.or.ctrl%mode.eq.4.or.ctrl%mode.eq.21).and.goback)&
  &goto 140 
   if(goback) goto 160
  !----------------------------------------------------------------
  ! check file containing total electronic energy 
  ! and try to read total electronic energy
  !----------------------------------------------------------------
  if(abs(ctrl%elec_energy).le.1.0d-10) then
    call clean(jobfile)
    jobfile = './job.last'
    call check_jobfile(jobfile)
    if (ctrl%mode.eq.3) then
      call get_energy_mp2(jobfile,ctrl%elec_energy,6)
!CMR can rimp2 handle open-shell cases? => total spin is not yet read
!CMR      call get_spin(jobfile,ctrl%spin,6)
    else
      call get_energy(jobfile,ctrl%elec_energy,6)
      call get_spin(jobfile,ctrl%spin,6)
    endif
  end if
  !----------------------------------------------------------------
  ! all input work done
  !----------------------------------------------------------------
  !----------------------------------------------------------------
  ! build up symmetry informations
  !----------------------------------------------------------------
  call create_char_table(char_table,ctrl%symmetry)
  call create_irred_atoms(molecule,char_table,atom_index,symm_index, &
       &                  irred_atom_index,nirred_atoms)
!MAXSYM>
  !----------------------------------------------------------------
  ! one distortion value for all cartesian coordinates
  ! note: generalized distortions may be necessary in order to
  !       perform distortions along the normal coordinates
  !----------------------------------------------------------------
  allocate(distortion(ubound(molecule,1)*3))
  distortion=ctrl%cstep
  if(ctrl%maxsym) then
     call create_irred_dist(molecule,char_table,dist_index, &
       &  dsymm_index,irred_dist_index,nirred_dist,nirred_atoms,&
       &  irred_atom_index,2*ctrl%numderiv,distortion,.true.)
  end if
!MAXSYM<
  !----------------------------------------------------------------
  ! warnings, consistency checks
  !----------------------------------------------------------------
  call def_warnings(ctrl,crap)
  !----------------------------------------------------------------
  ! write modified input files for turbomole, dalton, molpro ...
  ! Note: turbomole input files are always necessary
  !----------------------------------------------------------------
  if(associated(ctrl%tm_ctrl)) then
     call copy_ctrl_to_tm(ctrl)
     ctrl%tm_ctrl%symmetry='c1'
     call bfile(ctrl%tm_ctrl%control)
     call modify_tm_ctrl(ctrl%tm_ctrl)
  else
     call serror('snf_define',2,'Input-file generation failed')
  endif
  if(associated(ctrl%dal_ctrl)) then
    call copy_ctrl_to_dal(ctrl)
    call write_dal_ctrl(ctrl%dal_ctrl)
  end if
  !----------------------------------------------------------------
  ! write control, restart and snf_control
  !----------------------------------------------------------------
  if(ctrl%dynpol) ctrl_restart%dynpol=.true.
  if(associated(ctrl%dynfreq)) &
   &allocate(ctrl_restart%dynfreq(ubound(ctrl%dynfreq,1)))
  call read_restart(ctrl%restart,ctrl_restart,&
       & gradients,dipols,pols,istepfl,nvmach,status,ctrl%raman,6)
  if(status.ne.-9.and.ctrl_restart%numderiv.ne.orinum) then
    call serror('snf_define',0, &
  & 'Numderiv values different in control and restart')
    orinum=ctrl_restart%numderiv
  end if
  call destroy_control(ctrl_restart)
  wrestart=.true.
  if (status.eq.1 .or.status.eq.-1.or.status.eq.-2) then
     ! nonemtpy restart exists !!!
     valid= .false.
     wrestart=.false.
     do while(.not. valid)
        write(*,*)
        if(status.eq.1) then
           write(*,*)' Found restart file of a finished calculation'
        else if (status.eq.-1) then
           write(*,*)' Found restart file of a running or aborted calculation'
        else if (status.eq.-2) then
           call serror('snf_define',1,' Old restart file seems to be corrupt. ##&
            &Maybe you tried to calculate Raman intensities ##&
            &but restart file is not prepared for Raman.')
        endif
        write(*,*)
        write(*,*)' Would you like to write a new restart file? (y/n)'
        write(*,*)' (A backup of the existing restart file will be created.)'    
        write(*,*)' You might run into trouble if $scfconv has different'
        write(*,*)' values in control and restart.'
        write(*,*)
        write(*,*)' Note: it is always necessary to write a new restart'
        write(*,*)'       file in order to prepare a SNFDC run with'
        write(*,*)'       new settings !'
        write(*,*)
        write(*,*)' Default: n'
        write(*,*)
        read(*,'(a)') zeile
        if(len_trim(zeile).eq.0) then 
           valid = .true.
        elseif(len_trim(zeile) .eq. 1) then
           if(index('*yn',trim(zeile)).ne.0) then
              valid=.true.
           endif
        endif
     end do
     if(zeile.eq.'y') then
        call backup_restart(ctrl%restart,unit=6)
        wrestart=.true.
     endif
  endif
  if(wrestart) then
     ctrl%nvar=3*ubound(molecule,1)
     if(status.ne.0.and.status.ne.-9) then 
       allocate(oistepfl(ctrl%nvar*2*orinum))
       allocate(onvmach(ctrl%nvar*2*orinum))
       oistepfl(:)=istepfl(:)
       onvmach(:)=nvmach(:)
     end if
     if(associated(nvmach)) deallocate(nvmach)
     if(associated(istepfl)) deallocate(istepfl)
     call prepare_dummy_restart(irred_atom_index,nirred_atoms,ctrl,istepfl)
     if(ctrl%maxsym) call prepare_restart2(2*ctrl%numderiv,istepfl,&
          &                  irred_dist_index,ctrl%nvar/3,nirred_dist)
     if(orinum.ne.ctrl%numderiv.and.status.ne.0.and.status.ne.-9) then
     !-------------------------------------------------------------------------
     ! should old data be conserved ?
     !-------------------------------------------------------------------------
      valid=.false.
      do while(.not. valid)
        write(*,*)
        write(*,*)
        write(*,*)' Would you like to use data from old restart file ? (y/n)'
        write(*,*)
        write(*,*)' Default: y'
        write(*,*)
        read(*,'(a)') zeile
        if(len_trim(zeile).eq.0) then 
           valid = .true.
        elseif(len_trim(zeile) .eq. 1) then
           if(index('*yn',trim(zeile)).ne.0) then
              valid=.true.
           endif
        end if
      end do
      if(zeile(1:1).ne.'n') then
        allocate(nvmach(ctrl%nvar*2*ctrl%numderiv))
        allocate(ndipols(3,ctrl%nvar,2*ctrl%numderiv),&
            &   ngradients(ctrl%nvar,ctrl%nvar,2*ctrl%numderiv))
        ndipols=0.0d0
        ngradients=0.0d0
        if(ctrl%raman) then
          if(ctrl%dynpol.and.associated(ctrl%dynfreq)) then
            allocate(npols(8,ctrl%nvar,2*ctrl%numderiv,ubound(ctrl%dynfreq,1)))
          else
            allocate(npols(8,ctrl%nvar,2*ctrl%numderiv,1))
          end if
          npols=0.0d0
        end if
        !-------------------------------------------------------------------------
        ! copy corresponding entries in dipols, gradients, and pols
        !-------------------------------------------------------------------------
        do i=1,2*ctrl%numderiv
          ipre=2*ctrl%numderiv-(i-1)
          if(ipre.le.0) ipre=ipre-1
          do j=1,2*orinum
            ipreo=2*orinum-(j-1)
            if(ipreo.le.0) ipreo=ipreo-1
            if(ipre.eq.ipreo) then
              ndipols(:,:,i)=dipols(:,:,j)
              ngradients(:,:,i)=gradients(:,:,j)
              if(ctrl%raman) npols(:,:,i,:)=pols(:,:,j,:)
            end if
          end do
        end do
        if(status.ne.0) then
         do i=1,ctrl%nvar*2*ctrl%numderiv
          iiat=(i+(3*2*ctrl%numderiv-1))/(3*2*ctrl%numderiv)
          iidum=(i-3*2*ctrl%numderiv*(iiat-1))
          iixyz=(iidum+(2*ctrl%numderiv-1))/(2*ctrl%numderiv)
          iidiraus=mod(iidum,2*ctrl%numderiv)
          if(iidiraus.eq.0) iidiraus=2*ctrl%numderiv
          iidir=ctrl%numderiv-iidiraus+1
          if(iidir.le.0) iidir=iidir-1
          nvmach(i)='------'
          do j=1,ctrl%nvar*2*orinum
            jiat=(j+(3*2*orinum-1))/(3*2*orinum)
            jidum=(j-3*2*orinum*(jiat-1))
            jixyz=(jidum+(2*orinum-1))/(2*orinum)
            jidiraus=mod(jidum,2*orinum)
            if(jidiraus.eq.0) jidiraus=2*orinum
            jidir=orinum-jidiraus+1
            if(jidir.le.0) jidir=jidir-1
            if((iiat.eq.jiat).and.(iixyz.eq.jixyz).and.(iidir.eq.jidir)) then
              istepfl(i)=oistepfl(j)
              nvmach(i)=onvmach(j)
            end if
          end do
         end do
        end if
        !-------------------------------------------------------------------------
        ! deallocate old pointers and set new ones
        !-------------------------------------------------------------------------
        if(associated(dipols)) deallocate(dipols)
        if(associated(pols)) deallocate(pols)
        if(associated(gradients)) deallocate(gradients)
        dipols => ndipols
        gradients => ngradients
        if(ctrl%raman) pols => npols
      else
        if(associated(dipols)) deallocate(dipols)
        if(associated(pols)) deallocate(pols)
        if(associated(gradients)) deallocate(gradients)
      endif
     else 
      if(associated(dipols)) deallocate(dipols)
      if(associated(pols)) deallocate(pols)
      if(associated(gradients)) deallocate(gradients)
     end if
     call write_restart(gradients,dipols,pols,istepfl,nvmach,ctrl,unit=6)
  endif
  if(associated(istepfl)) deallocate(istepfl)
  if(associated(oistepfl)) deallocate(oistepfl)
  if(associated(nvmach)) deallocate(nvmach)
  if(associated(onvmach)) deallocate(onvmach)
  if(associated(gradients)) deallocate(gradients)
  if(associated(dipols)) deallocate(dipols)
  if(associated(pols)) deallocate(pols)
  !--------------------------------------------------------------------
  ! name of snf control file should always be "./snf_control", although
  ! input information may be read from other files.
  !--------------------------------------------------------------------
  ctrl%control='./snf_control'
  call write_control(ctrl,molecule)
200 continue
  call prepdc(crap)
  !----------------------------------------------------------------
  ! write reminder
  !----------------------------------------------------------------
  if(ctrl%symmetry .ne. 'c1'.and. associated(ctrl%tm_ctrl)) then
     write(*,*)
     write(*,*)' ********************************************************'
     write(*,*)'                     R E M I N D E R                     '
     write(*,*)' ********************************************************'
     write(*,*)'  Don''t forget to run define to get mos in c1-symmetry!'
     write(*,*)'    Check afterwards whether scfconv has been changed ! '
     write(*,*)' ********************************************************'
     write(*,*)
  else
     write(*,*)' good luck...'
  endif

  !----------------------------------------------------------------
  ! clean up memory
  !----------------------------------------------------------------
  call destroy_char_table(char_table)
  call destroy_molecule(molecule)
  call destroy_irred_atoms(atom_index,symm_index,irred_atom_index)
  if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)

contains
!************************************************************************

subroutine switchbox(u,val,key,descr)

  !----------------------------------------------------------------
  ! Utility function to print toggle-option
  !
  ! written by Carsten Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  integer,intent(in) :: u
  logical ,intent(in) :: val
  character(len=6),intent(in) :: key
  character(len=*),intent(in) :: descr
  if(val) then
     write(u,'(4x,a6,a18,(a))')key,' off : switch off ',trim(adjustl(descr))
  else
     write(u,'(4x,a6,a17,(a))')key,' on  : switch on ',trim(adjustl(descr))
  endif
end subroutine switchbox

!************************************************************************

subroutine getswitch(answer,val)
  !----------------------------------------------------------------
  ! Utility function to get toggle-option
  !
  ! written by Carsten Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  logical ,intent(inout) :: val
  character(len=*),intent(in) :: answer
  if(index(answer,'off').ne.0) val=.false.
  if(index(answer,'on' ).ne.0) val=.true.
end subroutine getswitch

!************************************************************************

  subroutine old_snfcont(ctrl,selatom,massatom,molecule,c,pointgrp)
  !----------------------------------------------------------------
  ! check, whether the user wants to use an old snf_control-file
  ! 
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
!  use f90_kind
!  use snf_control,        only: read_control,type_ctrl
!  use snf_turbomole,      only: read_tm_coord
!  use snf_error,          only: serror
!  use snf_irred_atoms,    only: get_point_grp,is_point_grp
!  use snf_molecule,       only: setisomass,type_atom,atom_equal
 
  implicit none
  type(type_ctrl),intent(inout) :: ctrl
  type(type_atom),pointer       :: molecule(:)
  logical                       :: valid
  character(len=132)            :: zeile
  character(len=3)              :: pointgrp
  character                     :: c
  integer,pointer               :: selatom(:)
  real(double),pointer          :: massatom(:)

  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' File snf_control already exists!'
     write(*,*)' Would you like to change the parameters (y/n)?'
     write(*,*)' (>return< = default = n)'
     write(*,*)
     read(*,'(a)')zeile
     zeile=trim(adjustl(zeile))
     if(len_trim(zeile).eq.0) then 
        valid = .true.
        c='n'
     elseif(len_trim(zeile) .le. 1) then
        if(index('yn',trim(zeile)).ne.0) then
           valid=.true.
           c=zeile(1:1)
        endif
     else
        write(*,*)
        write(*,*)' Unknown command'
        write(*,*)
     endif
  end do
  if(c.eq.'n') then
     !----------------------------------------------------------------
     ! take snf_control data and go to the end of interactive part
     !----------------------------------------------------------------
     call read_control(ctrl,selatom,massatom)
     if(associated(ctrl%tm_ctrl)) then
        call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
     else
       call serror('snf_define',2,'Only TURBOMOLE coord-files can be&
          & read so far !')
     endif
     !----------------------------------------------------------------
     ! test isosym, if specified
     !----------------------------------------------------------------
     if(ctrl%isosym.ne.'def') then
        write (*,*) 
        write (*,*) ' Checking molecular symmetry considering different isotopes'
        if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%isosym)) then
           call get_point_grp(molecule,1.D-3,atom_equal,pointgrp)
           write(*,*)
           write(*,*)'Point-group ',ctrl%isosym,' does not fit to coordinates.'
          write(*,*)'Automatic point-group check yields: ',pointgrp
           ctrl%isosym=pointgrp
        else
           write(*,*)' Symmetry: ',ctrl%isosym, ' is validated.'
        endif
     endif
     write (*,*) 
     write (*,*) ' Checking molecular symmetry. Ignoring different isotopes. '
     if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%symmetry)) then
        call get_point_grp(molecule,1.D-3,atom_equal,pointgrp)
        write(*,*)
        write(*,*)'Point-group ',ctrl%symmetry,' does not fit to coordinates.'
        write(*,*)'Automatic point-group check yields: ',pointgrp
        ctrl%symmetry=pointgrp
     else
        write(*,*)' Symmetry: ',ctrl%symmetry, ' is OK.'
     endif
     molecule(:)%changem=.false.
     if(associated(selatom)) call setisomass(molecule,selatom,massatom)
  endif
  end subroutine old_snfcont

!************************************************************************

  subroutine alt_cont(control,found_control)
  !----------------------------------------------------------------
  ! the user may enter the name of an alternative snf_control file
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  logical :: valid, found_control
  character(len=132) :: zeile,errbuff
  character(len=*)   :: control
  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' If you want to read default-data from another snf_control file,'
     write(*,*)' then enter its location/name or otherwise hit >return< '
     if(fconhere) write(*,*) ' to take data from snf_control as default values.'
     write(*,*)
     read(*,'(a)') zeile
     if(len_trim(zeile).eq.0) then 
        valid=.true.
     else
        control=trim(adjustl(zeile));
        found_control=.false.
        do i=1,len_trim(control)
           if(index('./',control(i:i)).eq.0) then
              inquire(file=control,exist=found_control)
              exit
           end if
        end do
        if(found_control) then
           valid=.true.
        else
           write(errbuff,'(2a)') 'Cannot find snf_control file:##',&
            &trim(adjustl(control))
           call serror('snf_define',0,errbuff)
           errbuff=''
        endif
     endif
  end do
  end subroutine alt_cont

!************************************************************************

  subroutine selprog(ctrl,molecule,goback)
  !----------------------------------------------------------------
  ! Select program to be used for the single point calculations
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  integer                 :: lw
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  character(len=132)      :: zeile
  logical valid,goback

  goback=.false.
  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' Which program would you like to use:'
     write(*,*)'   tm <path/control>     : TURBOMOLE'
     write(*,*)'   dalton <path/input>   : DALTON'
!     write(*,*)'   (molpro <path/input>) : MOLPRO (not yet implemented)'
     write(*,*)'   &                     : go back to control file selection'
     write(*,*)'   >return<              : quit menu and accept default settings' 
     write(*,*)'   (Default: tm control)'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(associated(ctrl%dal_ctrl)) deallocate(ctrl%dal_ctrl)
     if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
     if(len_trim(zeile).eq.0) then 
        allocate(ctrl%tm_ctrl)
        call init_tm_ctrl(ctrl%tm_ctrl)
        call read_tm_ctrl(ctrl%tm_ctrl)
        call copy_tm_to_ctrl(ctrl)
        call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
        ctrl%mode=ctrl%tm_ctrl%mode
        valid=.true.
     else if(zeile(1:2).eq.'tm') then
        lw = 2
        call gettmin(ctrl,molecule,valid,zeile,lw)
        ctrl%mode=ctrl%tm_ctrl%mode
        valid=.true.
     else if(zeile(1:6).eq.'dalton') then
!---------------------------------------------------------------------
! THIS IS ONLY FOR THE TURBOMOLE VERSION
!---------------------------------------------------------------------
        call serror('snf_define',0,'DALTON is not available &
       &in this version.')
        cycle
!---------------------------------------------------------------------
        allocate(ctrl%dal_ctrl)
        lw = 6
        call gettmin(ctrl,molecule,valid,zeile,lw)
        call init_dal_ctrl(ctrl%dal_ctrl)
        ctrl%mode = 20
        valid=.true.
     else if(zeile(1:6).eq.'molpro') then
        call serror('snf_define',0,'MOLPRO is still not implemented!##&
       &TURBOMOLE will be used instead.')
        lw = 6
        call gettmin(ctrl,molecule,valid,zeile,lw)
        ctrl%mode=ctrl%tm_ctrl%mode
        valid=.true.
     else if(zeile(1:1).eq.'&') then
        goback=.true.
        return
     end if
     ctrl%tm_ctrl%mode=ctrl%mode
  end do
  end subroutine selprog

!************************************************************************

  subroutine gettmin(ctrl,molecule,valid,zeile,lw)
  !----------------------------------------------------------------
  ! Get basic information from Turbomole control file
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  integer                 :: lw
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  character(len=132)      :: zeile,errbuff
  logical valid,extmc
  if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
  allocate(ctrl%tm_ctrl)
  if(len_trim(zeile).le.lw) then
     write(*,*)
     write(*,*) ' No input filename given, so default settings will be used'
     write(*,*) ' (Default: control)'
     write(*,*) 
     if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
     allocate(ctrl%tm_ctrl)
     call init_tm_ctrl(ctrl%tm_ctrl)
     call read_tm_ctrl(ctrl%tm_ctrl)
     call copy_tm_to_ctrl(ctrl)
     call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
     valid=.true.
  else
     extmc=.false.
     do i=3,len_trim(zeile((lw+1):))
        if(index('./',trim(adjustl(zeile(lw+i:lw+i)))).eq.0) then
           inquire(file=trim(adjustl(zeile((lw+1):))), exist=extmc)
           exit
        end if
     end do
     if (extmc) then
        call init_tm_ctrl(ctrl%tm_ctrl)
        ctrl%tm_ctrl%control=trim(adjustl(zeile((lw+1):)))
        call read_tm_ctrl(ctrl%tm_ctrl)
        call copy_tm_to_ctrl(ctrl)
        call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
        valid=.true.
     else
        write(errbuff,'(2a)') 'Cannot find TURBOMOLE control file:##',&
         &trim(adjustl(zeile((lw+1):)))
        call serror('snf_define',0,errbuff)
        errbuff=''
     end if
  end if
  end subroutine gettmin

!************************************************************************

  subroutine dalopt(ctrl,goback)
  !----------------------------------------------------------------
  ! Get basic information for DALTON control file
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl) :: ctrl
  logical         :: goback,valid,ex
  character(len=132) :: zeile,buf
  character(len=10) :: progtype,calctype
  valid=.false.
  goback=.false.
  do while(.not.valid)
    write(*,*)
    write(*,*)
    write(*,*) ' DALTON options: '
    write(*,*)
    call prog_type(ctrl%mode,progtype,calctype)
    if(index(progtype,'DALTON').eq.0) &
     & call serror('dalopt',2,'Wrong mode value for DALTON calculation')
    write(*,'(2a)') '    type of DALTON calculation: ',calctype
    if(index(ctrl%dal_ctrl%mos,'--------').eq.0) &
     & write(*,'(2a)') '    MOs in DALTON.MOPUN format will be taken &
     &form file: ',trim(adjustl(ctrl%dal_ctrl%mos))
    write(*,*)
    call switchbox(6,ctrl%dal_ctrl%tmmos,'tmmos ',&
     &'TURBOMOLE MOs as start vector for DALTON')
    write(*,*)'   mos <file> : supply MOs in DALTON.MOPUN format from <file>'
    write(*,*)'   <ctype>    : select type of calculation: '
    write(*,*)'                SCF, MP2, MCSCF, CC2, CCS or CCSD'             
    write(*,*)'   &          : go back to last menu'
    write(*,*)
    read(*,'(a)') zeile
    zeile=trim(adjustl(zeile))
    if(zeile(1:3).eq.'mos') then
      inquire(file=trim(adjustl(zeile(4:))),exist=ex)
      if(ex) then
        buf=trim(adjustl(zeile(4:)))
        read(buf,'(a)') ctrl%dal_ctrl%mos
        ctrl%dal_ctrl%tmmos=.false.
      else
        write(*,*) 
        write(*,*) ' No such file'
      end if
      cycle
    end if
    call lower(zeile)
    if((zeile(1:3).eq.'scf')) then
      ctrl%mode=20
    else if((zeile(1:3).eq.'mp2')) then
      ctrl%mode=21
    else if((zeile(1:5).eq.'mcscf')) then
      ctrl%mode=25
    else if((zeile(1:3).eq.'cc2')) then
      ctrl%mode=22
    else if((zeile(1:4).eq.'ccsd')) then
      ctrl%mode=24
    else if((zeile(1:3).eq.'ccs')) then
      ctrl%mode=23
    else if(len_trim(zeile).eq.0) then
      valid=.true.
    else if(zeile(1:5).eq.'tmmos') then
      call getswitch(zeile,ctrl%dal_ctrl%tmmos)
      if(ctrl%dal_ctrl%tmmos) ctrl%dal_ctrl%mos='----------'
    else if(zeile(1:1) .eq. '&') then
      goback=.true.
      return
    else
      write(*,*) 'Unknown command'
    end if
  end do
  
  end subroutine dalopt

!************************************************************************

  subroutine isomenu(ctrl,molecule,selatom,massatom,pointgrp,pgstat)
  !----------------------------------------------------------------
  ! select isotopes masses
  !
  ! written by J. Neugebauer and C. Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  integer,pointer         :: selatom(:)
  integer                 :: pgstat
  real(double),pointer    :: massatom(:)
  character(len=3)        :: pointgrp
  character(len=132)      :: zeile,zeile2,errbuff
  logical                 :: valid

  valid = .false.
11 do while(.not. valid)
     write(*,*)
     write(*,*)
     write(*,*) ' Isotope menu'
     write(*,*)
     write(*,*) ' If no isotopes are specified, masses of the most'
     write(*,*) ' abundant (for naturally occuring elements) or most'
     write(*,*) ' stable (for synthetical elements) isotopes will '
     write(*,*) ' be used to calculate the spectra.'
     write(*,*)
     if(any(molecule(:)%changem).or.associated(selatom)) then
        if(ctrl%isosym.eq.'def') then 
           call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
        else
           pointgrp=ctrl%isosym
        endif
        write(*,*) '***************************************************'
        write(*,*) ' WARNING! Specification of isotopes may change the'
        write(*,*) ' molecular symmetry. SNF will therefore perform'
        write(*,'(a,a5,a)') '  the calculations in ',pointgrp, &
             & ' symmetry. '
        if(ctrl%isosym.ne.'def') &
             write(*,*) ' THIS POINT-GROUP HAS BEEN SELECTED BY THE USER.'
        write(*,*) ' If you want to use another symmetry, then choose '
        write(*,*) ' command >sy<'
        write(*,*) '***************************************************'
        write(*,*)
     end if
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   >atom No.<       : specify an isotope for the selected atom'
     write(*,*)'   p                : print molecule information'
     write(*,*)'   d                : default settings'
     if(any(molecule(:)%changem).or.associated(selatom)) then
        write(*,*)'   sy >sym<         : set molecular symmetry to sym'
     end if
     write(*,*)'   >return<         : quit this menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(zeile(1:3).eq.'sy ') then
        if(any(molecule(:)%changem).or.associated(selatom)) then
           if(len_trim(zeile).gt.3) then
              pgstat=-1
              call lower(zeile(4:6))
              call checkpg(zeile(4:6),pgstat)
              if (pgstat.lt.1) then
                errbuff=''
                if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
                     zeile(4:6),'" is not implemented'
                if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
                     zeile(4:6)
                call serror('snf_define',0,errbuff)
              else
                 if(.not.is_point_grp(molecule,1.D-3,isotope_equal,zeile(4:6))) then
                    call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
                    write(*,*)
                    write(*,*)'Point-group ',zeile(4:6),' does not fit to coordinates.'
                    write(*,*)'Automatic point-group check yields: ',pointgrp
                    ctrl%isosym='def'
                 else
                    ctrl%isosym=zeile(4:6)
                 endif
              end if
           end if
        else
           write(*,*)' Unknown command'
        end if
     else if(zeile(1:1).eq.'p') then
        call print_molecule(molecule)
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for atomic masses.'
        write(*,*)
        do i=1,ubound(molecule,1)
           molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end do
        molecule(:)%changem=.false.
        ctrl%isosym='def'
        if(associated(selatom)) deallocate(selatom)
     else if(string2integer(zeile).gt.0.and.string2integer(zeile) &
          .le.ubound(molecule,1).and.(zeile(1:2).ne.'sy')) then
        nselatom=string2integer(zeile)
        if(molecule(nselatom)%element.eq."Q") then
          write(*,*) 
          write(*,*) ' You selected a dummy atom. I don''t think '
          write(*,*) ' you really want to change its mass ...  '
          goto 11
        end if
126     write(*,*) 
        write(*,'(a,i5)') ' Select mass number = number of nucleons for atom No. ',nselatom
        write(*,*)
        write(*,*) ' The following isotopes are available:'
        write(*,*)
        write(*,*) ' isotope:             atomic mass [amu]:'
        do i=1,3*(molecule(nselatom)%nucl_charge)
           if(fiso_mass(molecule(nselatom)%nucl_charge,i).gt.1.0d0) then
              write(*,'(2x,a,i3,a,i3,a,1x,a2,5x,f16.8)') &
                   '(',i,',',molecule(nselatom)%nucl_charge,')', &
                   molecule(nselatom)%element, &
                   fiso_mass(molecule(nselatom)%nucl_charge,i)
           end if
        end do
        write(*,*)
        write(*,*)' You may choose one of the following commands:'
        write(*,*)'   >atomic mass No.< : select isotope'
        write(*,*)'   m <mass>          : select another atomic mass [amu]'
        write(*,*)'   >return<          : go back to the main menu'
        write(*,*)
        read(*,'(a)') zeile2
        zeile2=trim(adjustl(zeile2))
        if(fiso_mass(molecule(nselatom)%nucl_charge,string2integer(zeile2)) &
             .gt.1.0d0) then
           molecule(nselatom)%mass= &
                fiso_mass(molecule(nselatom)%nucl_charge,string2integer(zeile2))
           molecule(nselatom)%changem=.true.
        else if (zeile2(1:1).eq.'m') then
           if(len_trim(zeile2).gt.1) then
              if(string2real(zeile2(2:)).ge.1.0d0)then
                 molecule(nselatom)%mass=string2real(zeile2(2:))
                 molecule(nselatom)%changem=.true.
              else 
                  call serror('snf_define',0,'Value for atomic mass &
                   &is critically small!')
              end if
           else
              write(*,*)
              write(*,*)'No <mass> given for atomic mass! Try again!'
              write(*,*)
              goto 126
           end if
        else if (len_trim(zeile2).ne.0) then
           if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
        end if
     else if(string2integer(zeile).gt.0.and.string2integer(zeile) &
          .gt.ubound(molecule,1)) then
        write(*,*)
        write(*,*)' No such atom in this molecule.'
        write(*,*)
     else if(len_trim(zeile).eq.0) then
        if(ctrl%isosym.ne.'def') then
           if(.not.is_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)) then
              call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
              write(*,*)
              write(*,*)'Point-group ',Ctrl%isosym,' does not fit to coordinates.'
              write(*,*)'Automatic point-group check yields: ',pointgrp
              Ctrl%isosym='def'
           else
              valid=.true.
           endif
        else
           valid=.true.
        end if
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     end if
  end do
  if(ctrl%isosym.eq.'def' .and.any(molecule(:)%changem)) then
     call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
     write(*,*)
     write(*,*)' Calcultion will be evaluated in ',pointgrp,' symmetry'
     write(*,*)' due to isotope settings.'
  endif
  if(associated(selatom)) then
     deallocate(selatom)
     deallocate(massatom)
  end if
  end subroutine isomenu

!************************************************************************

  subroutine nummenu(ctrl)
  !----------------------------------------------------------------
  ! Numerical derivative menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile
  logical            :: valid
  valid = .false.
  do while(.not. valid)
     write(*,*)
     write(*,*) ' Numerical derivative menu'
     write(*,*)
     write(*,'(''  Current settings: num='',i1,''  cstep='',f6.3)') &
      &  ctrl%numderiv*2+1,ctrl%cstep
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   num 3|5     : set 3-point or 5-point formula '
     write(*,*)'   cstep <val> : set step parameter on <val> [bohr] '
     write(*,*)'   d           : default settings'
     write(*,*)'   >return<    : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if((zeile(1:3)).eq.'num') then
        if(nint(string2real(zeile(4:))).eq.3.or.nint(string2real(zeile(4:))).eq.5) then
           ctrl%numderiv=nint(string2real(zeile(4:)))
           ctrl%numderiv=(ctrl%numderiv-1)/2
        else
           if(len_trim(zeile).le.3) then
              write(*,*)
              write(*,*)' No <val> given for num! Try again!'
              write(*,*)
           else
              write(*,*)
              write(*,*)' Only 3 and 5 are acceptable values for num! Try again!'
              write(*,*)
           end if
        end if
     else if(zeile(1:5).eq.'cstep') then
        if(len_trim(zeile).gt.5) then
           if(string2real(zeile(6:)).ge.cthres)then
              ctrl%cstep=string2real(zeile(6:))
           else 
              call serror('snf_define',0,'Value for cstep is critically &
               &small! Try again!')
           end if
        else
           write(*,*)
           write(*,*)'No <val> given for cstep! Try again!'
           write(*,*)
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for num and cstep.'
        write(*,*)
        ctrl%cstep=0.01
        ctrl%numderiv=1
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  end do
  end subroutine nummenu

!************************************************************************

  subroutine specmenu(ctrl)
  !----------------------------------------------------------------
  ! Spectrum settings menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid
  character(len=132) :: zeile

  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' Spectrum settings menu'
     write(*,*)
     write(*,'(''  Current settings: minf='',f6.1,''   maxf='',f6.1,''   w='',f5.1,''   np='',i6)') &
          &    ctrl%minfreq,ctrl%maxfreq,ctrl%width,ctrl%npoint
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   minf <val> : set lower frequency bound in cm^(-1)' 
     write(*,*)'   maxf <val> : set upper frequency bound in cm^(-1)'
     write(*,*)'   w    <val> : set peak width in gaussian represenation in cm^(-1)'
     write(*,*)'   np   <num> : set number of grid points for gaussian representation '
     write(*,*)'   d          : default settings'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if(zeile(1:4).eq.'minf') then
        if(string2real(zeile(5:)).lt.0.) then
           write(*,*)
           write(*,*)' minf must be greater than or equal zero!'
           write(*,*)
        else if(string2real(zeile(5:)).ge.ctrl%maxfreq) then
           write(*,*)
           write(*,*)' minf must be less than maxf!'
           write(*,*)
        else
           ctrl%minfreq=string2real(zeile(5:))
        end if
     else if(zeile(1:4).eq.'maxf') then
        if(string2real(zeile(5:)).le.ctrl%minfreq) then
           write(*,*)
           write(*,*)' maxf must be greater than minf!'
           write(*,*)
        else
           ctrl%maxfreq=string2real(zeile(5:))
        end if
     else if(zeile(1:1).eq.'w') then
        if(string2real(zeile(2:)).le.0.0d0) then
           write(*,*)
           write(*,*)' width must be greater than zero!'
           write(*,*)
        else
           ctrl%width=string2real(zeile(2:))
        end if
     else if(zeile(1:2).eq.'np') then
        if(nint(string2real(zeile(3:))).le.0.) then
           write(*,*)
           write(*,*)' np must be greater than zero!'
           write(*,*)
        else
           ctrl%npoint=nint(string2real(zeile(3:)))
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for minf, maxf, width and np.'
        write(*,*)
        ctrl%minfreq=200
        ctrl%maxfreq=3500
        ctrl%width=10
        ctrl%npoint=2000
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine specmenu

!************************************************************************

  subroutine thermmenu(ctrl)
  !----------------------------------------------------------------
  ! thermodynamics menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid
  character(len=132) :: zeile

  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)'Thermodynamics menu'
     write(*,*)
     write(*,'(''  Current settings:  t='',f5.1,'' k    p='',f10.4,&
       &'' bar'')')ctrl%temp,ctrl%press/100000.0d0
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   t <val>   : temperature in k'
     write(*,*)'   p <val>   : pressure in bar'
     write(*,*)'   d         : default settings'
     write(*,*)'   >return<  : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if(zeile(1:1).eq.'t') then
        if(string2real(zeile(2:)).lt.0.) then
           write(*,*)
           write(*,*)' Temperature must be greater than or equal zero!'
           write(*,*)
        else
           ctrl%temp=string2real(zeile(2:))
        end if
     else if(zeile(1:1).eq.'p') then
        if(string2real(zeile(2:))*100000.0d0.lt.0) then
           write(*,*)
           write(*,*)' Pressure must be greater than or equal zero!'
           write(*,*)
        else
           ! conversion from bar to pa
           ctrl%press=string2real(zeile(2:))*100000.0d0
           ! test on pressure dimension
           if (ctrl%press.gt.1.0d8) then
              write(*,*)
              write(*,*) "******************************************"
              write(*,*) " WARNING: pressure must be given in [bar]!"
              write(*,*) "******************************************"
              write(*,*)
           end if
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for t and p.'
        write(*,*)
        ctrl%temp=273.15d0+25.0d0
        ctrl%press=101325.0d0
     else if(len_trim(zeile).eq.0) then   
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine thermmenu

!************************************************************************

  subroutine ramanmenu(ctrl,goback)
  !----------------------------------------------------------------
  ! Raman intensities menu
  !
  ! written J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile
  character(len=3)   :: escftxt
  character(len=10)  :: ptype,ctype
  logical            :: valid,goback,tmval,riflag
  integer            :: oldmode
  valid = .false.
  goback= .false.
  riflag=.false.
  oldmode=ctrl%mode
  if(oldmode.eq.2.or.oldmode.eq.7.or.oldmode.eq.8) riflag=.true.
  escftxt='std'
  do while(.not. valid)
     write(*,*)
     write(*,*)' Raman intensities menu'
     write(*,*)
     write(*,*)'  CAUTION: Calculation of Raman intensities'
     write(*,*)'           is very time-consuming !'
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     call switchbox(6,ctrl%raman,'raman ','Raman intensity calculation')
     if(ctrl%raman) then
       call switchbox(6,ctrl%dynpol,'dynpol',&
        &'dynamic polarizabilities for Raman intensities')
       call prog_type(ctrl%mode,ptype,ctype)
       tmval=(index(ptype,'TURBOMOLE').ne.0)
       if(tmval) then
         write(*,*)'   escf int   : use escf.intel working version'
         write(*,*)'   escf ath   : use escf.athlon working version'
         if(.not.riflag) then
           write(*,*)'   escf std   : use escf standard version'
         else if(ctrl%mode.eq.2) then
           ctrl%mode=7
         end if
         if(ctrl%mode.eq.1) escftxt='std'
         if(ctrl%mode.eq.5) escftxt='int'
         if(ctrl%mode.eq.6) escftxt='ath'
         if(ctrl%mode.eq.7) escftxt='int'
         if(ctrl%mode.eq.8) escftxt='ath'
         write(*,'(2a)') '                 current setting: ',escftxt
       end if
     end if
     if(ctrl%raman.and.ctrl%dynpol) then
       write(*,*)'   wl <val>   : enter up to 10 different wavelengths [nm]'
       write(*,*)'                for calculation of polarizabilities'
       write(*,*)'   fr <val>   : enter up to 10 different frequencies [a.u.]'
       write(*,*)'                for calculation of polarizabilities'
       write(*,*)'   p          : print choosen frequencies (default = 514.5 nm)'
     end if
     write(*,*)'   d          : default settings'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if((zeile(1:5).eq.'raman')) then
        call getswitch(zeile,ctrl%raman)
     else if((zeile(1:6).eq.'dynpol').and.ctrl%raman) then
        call getswitch(zeile,ctrl%dynpol)
     else if((zeile(1:2).eq.'fr').and.ctrl%raman.and.ctrl%dynpol) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl)
     else if((zeile(1:2).eq.'wl').and.ctrl%raman.and.ctrl%dynpol) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl)
        if(associated(ctrl%dynfreq)) then
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              call serror('snf_define',0,'Infinite frequency detected!##&
               &Set frequency to 0.0d0 instead.')
            else
              ctrl%dynfreq(i)=nm2freq/ctrl%dynfreq(i)
            end if
          end do
        end if
     else if((zeile(1:5).eq.'escf ').and.ctrl%raman.and.tmval) then
       if(zeile(6:8).eq.'int') then
         if(riflag) then
           ctrl%mode=7
         else 
           ctrl%mode=5
         end if
       else if(zeile(6:8).eq.'ath') then
         if(riflag) then
           ctrl%mode=8
         else 
           ctrl%mode=6
         end if
       else if(zeile(6:8).eq.'std') then
         if(riflag) then
           write(*,*) 'Sorry, it is not possible to use ridft with &
           &standard escf'
         else
           ctrl%mode=1
         end if
       end if
       ctrl%tm_ctrl%mode=ctrl%mode
     else if(len_trim(zeile).eq.0) then 
        valid = .true.
     else if(zeile(1:1) .eq. 'p'.and.len_trim(zeile).eq.1.and.ctrl%raman.and.&
       &ctrl%dynpol) then
        write (*,*) 
        write (*,*) ' selected wavelengths/frequencies for dynamical polarizabilities:'
        write (*,*) 
        if(associated(ctrl%dynfreq)) then
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              write(*,'(a,a,f16.8,a)') '    wavelength(1) =         infinite',&
               &' nm ; angular frequency = ',0.0d0, ' a.u.'
            else
             write(*,'(a,i1,a,f16.8,a,f16.8,a)') '    wavelength(',i,') = ',&
              &1.d0/ctrl%dynfreq(i)*nm2freq, ' nm ; angular frequency = ',&
              &ctrl%dynfreq(i),' a.u.'
            end if
          end do
        else
          write(*,'(a,f16.8,a,f16.8,a)') '    wavelength(1) = ',514.5,&
           &' nm ; angular frequency =  ',nm2freq/514.5, ' a.u.'
        end if
        write (*,*) 
     else if(zeile(1:1) .eq. 'd'.and.len_trim(zeile).eq.1) then
        ctrl%mode=oldmode
        ctrl%raman=.false.
        ctrl%dynpol=.false.
        if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
     else if(zeile(1:1) .eq. '&') then
       goback=.true.
       return
     else 
        write(*,*)
        write(*,*)' Unknown command'
     endif
  end do
  end subroutine ramanmenu

!************************************************************************

  subroutine outmenu(ctrl,goback)
  !----------------------------------------------------------------
  ! output menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile
  logical            :: valid,goback,all
  valid=.false.
  goback=.false.
  all=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' Output options menu'
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     call switchbox(6,ctrl%print_irred_at,'iat   ','information about symmetry redundant atoms')
     call switchbox(6,ctrl%prirdist,'idt   ','information about symmetry redundant distortions')
     call switchbox(6,ctrl%print_hessian,'hess  ','output of cartesian hessian')
     call switchbox(6,ctrl%sumdeg,'sumdeg','sum intensities for degenerate modes')
     if(ctrl%raman) call switchbox(6,ctrl%print_rcs,'rcs   ','output of raman cross sections')
     call switchbox(6,ctrl%print_gaussian,'gau   ','gaussian fake output')
     call switchbox(6,ctrl%print_mopac,'mopac ','mopac fake output')
     if(ctrl%print_mopac) call switchbox(6,ctrl%print_mopxmol,'moxmol',&
       &'mopac fake output readable by xmol')
     call switchbox(6,ctrl%print_xmol,'xmol  ','output of xmol XYZ input file')
     if(all) then
       call switchbox(6,ctrl%print_symmetry,'sym   ','general symmetry informations')
       call switchbox(6,ctrl%print_cioslowski,'cio   ',&
        &'cioslowski charge decomposition')
       call switchbox(6,ctrl%print_zpe,'zpe   ','zeropoint energy output')
       call switchbox(6,ctrl%print_thermo,'thermo','detailed thermodynamic output')
       call switchbox(6,ctrl%print_transrot,'trm   ',&
        & 'translational and rotational contributions output')
       call switchbox(6,ctrl%print_force,'force ','output of force constant matrix')
       call switchbox(6,ctrl%print_dip,'dip   ','output of cartesian dipol derivates')
       call switchbox(6,ctrl%tm_ctrl%tmpdirclean,'tmpcl ','removal of temporary directories for single point calculations')
       call switchbox(6,ctrl%tm_ctrl%logdirclean,'logcl ','removal of logfiles for slave processes')
       if(ctrl%raman) call switchbox(6,ctrl%print_pol,'pol   ', &
         & 'output of cartesian polarizability derivates')
     end if
     write(*,*)'   bak        : create backup copy of original mos file'
     if(.not.all) write(*,*)'   all        : show all options'
     if(all) write(*,*)'   all off    : show reduced set of options'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if((zeile(1:3).eq.'sym')) then
        call getswitch(zeile,ctrl%print_symmetry)
     else if((zeile(1:3)).eq.'iat') then
        call getswitch(zeile,ctrl%print_irred_at)
     else if((zeile(1:3)).eq.'idt') then
        call getswitch(zeile,ctrl%prirdist)
     else if((zeile(1:3)).eq.'cio') then
        call getswitch(zeile,ctrl%print_cioslowski)
     else if((zeile(1:5)).eq.'force') then
        call getswitch(zeile,ctrl%print_force)
     else if((zeile(1:4)).eq.'hess') then
        call getswitch(zeile,ctrl%print_hessian)
     else if((zeile(1:3)).eq.'dip') then
        call getswitch(zeile,ctrl%print_dip)
     else if((zeile(1:6)).eq.'sumdeg') then
        call getswitch(zeile,ctrl%sumdeg)
     else if((zeile(1:3)).eq.'pol') then
        call getswitch(zeile,ctrl%print_pol)
     else if((zeile(1:3)).eq.'rcs') then
        call getswitch(zeile,ctrl%print_rcs)
     else if((zeile(1:3)).eq.'trm') then
        call getswitch(zeile,ctrl%print_transrot)
     else if((zeile(1:3)).eq.'zpe') then
        call getswitch(zeile,ctrl%print_zpe)
     else if((zeile(1:6)).eq.'thermo') then
        call getswitch(zeile,ctrl%print_thermo)
     else if((zeile(1:3)).eq.'gau') then
        call getswitch(zeile,ctrl%print_gaussian)
     else if((zeile(1:4)).eq.'xmol') then
        call getswitch(zeile,ctrl%print_xmol)
     else if((zeile(1:5)).eq.'tmpcl') then
        call getswitch(zeile,ctrl%tm_ctrl%tmpdirclean)
     else if((zeile(1:5)).eq.'logcl') then
        call getswitch(zeile,ctrl%tm_ctrl%logdirclean)
     else if((zeile(1:5)).eq.'mopac') then
        call getswitch(zeile,ctrl%print_mopac)
     else if((zeile(1:6)).eq.'moxmol') then
        call getswitch(zeile,ctrl%print_mopxmol)
     else if((zeile(1:3)).eq.'bak') then
        call bfile('mos')
     else if((zeile(1:14)).eq.'werhatsgemacht') then
        call pr_snfdefhead
     else if((zeile(1:7)).eq.'all off') then
        all=.false.
     else if((zeile(1:3)).eq.'all') then
        all=.true.
     else if((zeile(1:1)).eq.'&') then
        goback=.true.
        return
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine outmenu

!************************************************************************

  subroutine check_jobfile(jobfile)
  !----------------------------------------------------------------
  ! check file containing total electronic energy 
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  character(len=*) :: jobfile
  character(len=132) :: zeile
  logical           :: ex
  inquire(file=jobfile, exist=ex)
  if(.not.ex) then
    write(*,*)
    write(*,'(3a)') '  Cannot find file: ',jobfile(1:len_trim(jobfile)),&
     &' to read electronic energy.'
    write(*,*) ' (This is only necessary to get the correct total energy.)'
    write(*,*) ' You may enter an alternative filename to read the &
     &electronic energy from'
    write(*,*) ' (otherwise it is set to zero).'
    write(*,*)
    read(*,'(a)') zeile
    if(.not.(len_trim(zeile).eq.0)) then
      jobfile=''
      write(jobfile,'(a)') zeile(1:len_trim(zeile))
      write(*,*)
      write(*,'(2a)') '  Trying to read total energy from file: ',&
       &trim(adjustl(jobfile))
    end if
  end if
  end subroutine check_jobfile

!************************************************************************

  subroutine def_warnings(ctrl,crap)
  !----------------------------------------------------------------
  ! warnings, consistency checks
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid,crap
  character(len=132) :: zeile

  if (ctrl%scfconv.lt.8) then
    valid=.false.
    do while(.not. valid)
      write(*,*)
      write(*,'(a,i1)')'  Warning: $scfconv = ',ctrl%scfconv
      write(*,*)' We strongly recommend $scfconv > 7 !'
      write(*,*)
      write(*,*)'   Enter $scfconv-value to be used in this calculation'
      write(*,*)'   or press return to set default value ($scfconv = 8).'
      write(*,*)
      read(*,'(a)') zeile
      zeile=trim(adjustl(zeile))
      if(string2integer(zeile).gt.0.and.string2integer(zeile).le.10) then
        ctrl%scfconv = string2integer(zeile)
        valid = .true.
      else if(string2integer(zeile).gt.10) then
        write(*,*)
        write(*,*)' $scfconv too large'
        write(*,*)
      else if(zeile.eq.'') then
        ctrl%scfconv = 8
        valid = .true.
      end if
    end do
  endif
  if(ctrl%cstep.lt.0.005) then
     call serror('snf_define',0,'ctrl%cstep is critically small !!!')
  end if
  crap=.false.
  if(ctrl%cstep.lt.0.01.or.ctrl%cstep.gt.0.05.or.ctrl%scfconv.lt.8) then
     crap=.true.
     write(*,*)
     write(*,*) ' *************************************************************'
     write(*,*)'   Note: <CRAP_OK yes> must be set in DCINPUT input,'
     if(ctrl%cstep.lt.0.01) then
        write(*,*)'         since cstep < 0.01 !!!'
     else if(ctrl%cstep.gt.0.05) then
        write(*,*)'         since cstep > 0.05 !!!'
     else if(ctrl%scfconv.lt.8) then
        write(*,*)'         since $scfconv < 8 !!!'
     end if
     write(*,*) ' *************************************************************'
     write(*,*)
  end if
  if(ctrl%raman) then
    select case(ctrl%mode)
    case(2)
    ! ridft not available for Raman calculation with ESCF V5.4
      call serror('snf_define',0,'No ridft available for Raman calculations##&
       & using ESCF V5.4. Switching to ESCF working version escf.intel!')
      ctrl%mode=7
      ctrl%tm_ctrl%mode=7
    case(3:4,21)
    ! no rimp2, mpgrad or Dalton MP2 for Raman
      call serror('snf_define',0,'Raman calculations not available for this##&
       & program mode. Raman is switched off')
      ctrl%raman=.false.
    case(7:8)
    ! ridft has problems with dynamic polarizabilities
      if(ctrl%dynpol) then
        call serror('snf_define',0,'Note that dynamic polarizabilities##&
         & might cause serious problems in combination with ridft!')
      end if
    end select
  end if
  end subroutine def_warnings

!************************************************************************
  subroutine prepdc(crap)

!-----------------------------------------------------------------------
! prepare inputfiles USE_NODES and DCINPUT for SNFDC.
! furthermore, the files MACHINES and mpi.sub are
! created, which may be used for the MPI version of SNFDC.
!
! written by J. Neugebauer, Uni Erlangen, 2002
!-----------------------------------------------------------------------
  implicit none
  integer info,system,info2,li,ncpu
  character(len=255) buf
  logical crap
  parameter(li=1)
  write(*,*) '... creating file MACHINES for MPI-MACHINERY'
  info=system('choose_nodes')
  if(info.ne.0) call serror('prepdc',1,'choose_nodes failed! Make &
  &sure that choose_nodes is available in your path!')
  ncpu=0
  open(unit=li,file='USE_NODES')
  do i=1,3
    read(li,'(A)',end=999) buf
  end do
10 read(li,'(A)',end=999) buf
  if(index(buf,'# chosen').ne.0) goto 20
  goto 10
999 call serror('prepdc',1,'Could not determine number of nodes !')
  goto 30
20 read(buf(index(buf,'with')+4:index(buf,'proc')-1),*) ncpu
30 close(li)
  info=system('choose_nodes -mpi')
  info2=system('mv USE_NODES MACHINES')
  if(info.ne.0.or.info2.ne.0) call serror('prepdc',1,'choose_nodes &
  &failed! Make sure that choose_nodes is available in your path!')
  write(*,*) '... creating file mpi.sub for MPI-MACHINERY'
  open(unit=li,file='mpi.sub')
!---------------------------------------------------------------------------
!  write(li,'(A,i3,A)') '$MPICH/bin/mpirun -machinefile MACHINES -np ',ncpu,&
!  &' snfdcmpi'
  write(li,'(A,i3,A)') 'mpirun -machinefile MACHINES -np ',ncpu,&
  &' snfdc'
!---------------------------------------------------------------------------
  close(li)
  info=system('chmod u+x mpi.sub')
  if(info.ne.0) call serror('prepdc',1,'Error while preparing mpi.sub')
  write(*,*) 'Note that it might be necessary to change the options in mpi.sub'
  write(*,*) '... creating file USE_NODES for PVM-MACHINERY'
  info=system('choose_nodes')
  if(info.ne.0) call serror('prepdc',1,'choose_nodes failed! Make &
  &sure that choose_nodes is available in your path!')
  write(*,*) '... creating file DCINPUT:'
  info=system('mkdcinput 2>/dev/null')
  info2=0
  if(crap) info2=system('echo CRAP_OK yes >> DCINPUT')
  if(info.ne.0.or.info2.ne.0) call serror('prepdc',1,'mkdcinput failed! Make &
  &sure that mkdcinput is available in your path!')
  write(*,*) 
  write(*,*) 'DCINPUT: '
  write(*,*) 
  info=system('cat DCINPUT')
  write(*,*) 
  write(*,*) 'change the parameters in DCINPUT according to your &
  &needs if necessary'
  write(*,*) 
  end subroutine prepdc

!************************************************************************

subroutine pr_snfdefhead
  implicit none
  write(*,*)
  write(*,*)' *************************************************************'
  write(*,*)' *                                                           *'
  write(*,*)' *                     SNFDEFINE V2.1.4                      *'
  write(*,*)' *                                                           *'
  write(*,*)' *                    (TURBOMOLE VERSION)                    *'
  write(*,*)' *                                                           *'
  write(*,*)' *      C. Kind, M. Reiher, J. Neugebauer, and B.A. Hess     *'
  write(*,*)' *                                                           *'
  write(*,*)' *              University of Erlangen 1999-2002             *'
  write(*,*)' *                                                           *'
  write(*,*)' *************************************************************'
  write(*,*)
end subroutine pr_snfdefhead

!************************************************************************

end program snfdefine
