
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
      module snf_dummy
  !*********************************************************************
  ! prepares evaluation of coord and restart files containing dummy
  ! atoms for SNF.
  !
  ! written by J. Neugebauer, Uni Erlangen, 10/2001
  !*********************************************************************
      use f90_kind
      use snf_error
      use snf_molecule,   only: type_atom
 
      private
      public :: rm_dummies

  !*********************************************************************

      contains

  !*********************************************************************
      subroutine rm_dummies(molecule,grad,dip,pol,nvmach,istepfl,&
     & naus,ramanfl,nfreq)
  !---------------------------------------------------------------------
  ! remove dummy atoms and the corresponding entries in grad,dip,pol,
  ! nvmach, istepfl. Deallocate old pointers and assign to new targets
  !---------------------------------------------------------------------
      implicit none
      type(type_atom),pointer  :: molecule(:),molecule2(:)
      logical,intent(in)       :: ramanfl
      integer,intent(in)       :: naus,nfreq
      integer,pointer          :: istepfl(:),istepfl2(:)
      logical,pointer          :: dumfl(:)
      character(len=8),pointer :: nvmach(:),nvmach2(:)
      real(double),pointer     :: grad(:,:,:),dip(:,:,:),pol(:,:,:,:)
      real(double),pointer     :: grad2(:,:,:),dip2(:,:,:),pol2(:,:,:,:)
      integer                  :: i,natoms,count,c2,nv,j,nnatoms
  
      natoms=ubound(molecule,1)
      allocate(dumfl(natoms))
      dumfl=.true.
      !----------------------------------------------------------------
      ! determine no. of atoms, neglecting dummies
      !----------------------------------------------------------------
      nnatoms=0
      do i=1,natoms
        if(molecule(i)%element.eq.'Q') then
          dumfl(i)=.false.
          cycle
        end if
        nnatoms=nnatoms+1
      end do
      nv=3*nnatoms

      allocate(molecule2(nnatoms),istepfl2(nv*naus),nvmach2(nv*naus))
      allocate(grad2(nv,nv,naus),dip2(3,nv,naus))
      if(ramanfl) allocate(pol2(8,nv,naus,nfreq))
      !-----------------------------------------------------------------
      ! copy necessary data to new pointers
      !-----------------------------------------------------------------
      count=0
      do i=1,natoms
        if(dumfl(i)) then
          count=count+1
          molecule2(count)=molecule(i)
          istepfl2((count-1)*3*naus+1:count*3*naus)=&
     &       istepfl((i-1)*3*naus+1:count*3*naus)
          nvmach2((count-1)*3*naus+1:count*3*naus)= &
     &       nvmach((i-1)*3*naus+1:count*3*naus)
          dip2(:,(count-1)*3+1:count*3,:)=dip(:,(i-1)*3+1:i*3,:)
          c2=0
          do j=1,natoms
            if(dumfl(j)) then
              c2=c2+1
              grad2((c2-1)*3+1:c2*3,(count-1)*3+1:count*3,:)=&
     &         grad((j-1)*3+1:j*3,(i-1)*3+1:i*3,:)
            end if
          end do
          if(ramanfl) &
     &      pol2(:,(count-1)*3+1:count*3,:,:)=pol(:,(i-1)*3+1:i*3,:,:)
        end if
      end do

      !-----------------------------------------------------------------
      ! deallocate old pointers and assign pointers to new targets
      !-----------------------------------------------------------------
      deallocate(grad,dip,molecule,istepfl,nvmach)
      grad => grad2
      dip => dip2
      molecule => molecule2
      istepfl => istepfl2
      nvmach => nvmach2
      if(ramanfl) then
        deallocate(pol)
        pol => pol2
      end if

      end subroutine rm_dummies
  !*********************************************************************

      end module snf_dummy
