
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_error

  private
  public :: serror

contains

  subroutine serror(moduleori,ierr,errmsgori,unit)
    !----------------------------------------------------------------
    ! error management
    ! written by J. Neugebauer, Uni Erlangen, 2001
    !
    ! errmsg : error message
    !
    ! ierr   : ierr = 0 : warning;
    !          ierr = 1 : error (without stop)
    !          ierr = 2 : fatal error, program stops
    !          otherwise: message
    !
    ! module : name of module (subprogram, function), where the
    !          warning / error occured
    !
    ! NOTE: If error messages are longer than just one line,
    !       '##' may be used to separate the lines (i.e. '##'
    !       means <return>).
    !----------------------------------------------------------------
    implicit none
    integer, parameter :: maxerrl=500
    integer, parameter :: maxml=50
    character(len=*) :: errmsgori,moduleori
    character(len=132),pointer:: errrow(:)
    character(len=maxerrl) :: errmsg
    character(len=maxml) :: module
    character(len=9) :: msgtyp
    character(len=5) :: lhead,cons
    character(len=2) :: newln
    integer, optional :: unit
    integer i,l1,l2,out,linel,diff1,diff2,l3,lend,j,start
    integer ierr
    integer, pointer :: diff(:)
    parameter ( lhead = ' *** ' )
    parameter ( newln = '##' )
    if(len_trim(errmsg).gt.maxerrl) then
      write(*,*) 
      write(*,*) '*************************************************'
      write(*,*) '*** WARNING! from module snf_error            ***'
      write(*,*) '*** Error message is truncated since too long ***'
      write(*,*) '*************************************************'
      write(*,*) 
    else if(len_trim(module).gt.maxml) then
      write(*,*) 
      write(*,*) '********************************************'
      write(*,*) '*** WARNING! from module snf_error       ***'
      write(*,*) '*** Module name truncated since too long ***'
      write(*,*) '********************************************'
      write(*,*) 
    end if
    if ( ierr .eq. 0 ) then
      msgtyp = 'WARNING! '
      cons = 'from '
    elseif (ierr.eq.1.or.ierr.eq.2) then
      msgtyp = 'ERROR!!! '
      cons = 'in '
    else
      msgtyp = 'MESSAGE! '
      cons = 'from '
    endif
    module=trim(adjustl(moduleori))
    errmsg=trim(adjustl(errmsgori))
    i=0
    start=1
    do while (index(errmsg(start:),newln).ne.0)
      i=i+1
      start=index(errmsg(start:),newln)+1+start
    end do
    if(i.ne.0) allocate(errrow(i))
    i=0
    do while (index(errmsg,newln).ne.0) 
      i=i+1
      errrow(i)=errmsg(1:index(errmsg,newln)-1)
      lend=len_trim(errmsg)
      errmsg=trim(adjustl(errmsg(index(errmsg,newln)+2:lend)))
    end do
    l1=len_trim(module)
    l2=len_trim(errmsg)

    if(i.ne.0) then
      allocate(diff(i))
      l2=len_trim(errrow(1))
      do i=2,ubound(diff,1)
        l2=max(l2,len_trim(errrow(i)))
      end do
      l2=max(l2,len_trim(errmsg))
    end if
    l3=len_trim(cons)
    linel=max(25+l3+l1,8+l2)
    diff1=linel-25-l3-l1
    diff2=linel-8-len_trim(errmsg)
    if(associated(diff)) then
      do i=1,ubound(diff,1)
        diff(i)=linel-8-len_trim(errrow(i))
      end do
    end if
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'(1x,280a1)') ('*',i=1,linel)
    write(out,100) lhead,msgtyp,trim(adjustl(cons)),' module ',&
     & trim(adjustl(module)),(' ',i=1,diff1),lhead
    if(associated(diff)) then
      do j=1,ubound(diff,1)
        write(out,'(150a)') lhead,trim(adjustl(errrow(j))),(' ',i=1,diff(j)),lhead
      end do
    end if
    write(out,'(150a)') lhead,trim(adjustl(errmsg)),(' ',i=1,diff2),lhead
    write(out,'(1x,180a1)') ('*',i=1,linel)
    write(out,*)
    if(associated(diff)) deallocate(diff)
    if(associated(errrow)) deallocate(errrow)
    if(ierr.eq.2) stop
100 format(a5,a9,a,a,150a)
  end subroutine serror

end module snf_error








