
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_force
  use f90_kind

  private
  public :: hessian_mass_weighting,write_tm_force,print_force_matrix
  public :: calc_atom_force
  !************************************************************************

contains

  !************************************************************************


  subroutine hessian_mass_weighting(hessian_xyz,hessian_m,molecule)
    use snf_molecule, only: type_atom
    implicit none      
    type(type_atom),intent(in) :: molecule(:)
    real(double), intent(in) :: hessian_xyz(:,:)
    real(double), intent(out) :: hessian_m(:,:)
    integer :: i,j,nvar
    !----------------------------------------------------------------
    ! mass weighting of hessian 
    !                      hessian_xyz(i,j)
    ! hessian_xyz(i,j) = ------------------- 
    !                    sqrt(m_i)*sqrt(m_j)
    !
    ! unit is hartree*mol/(bohr^2*g)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    nvar=ubound(hessian_xyz,1)
    do j=1,nvar
       do i=1,j
          hessian_m(i,j)=hessian_xyz(i,j)&
               & /sqrt(molecule((i+2)/3)%mass*molecule((j+2)/3)%mass)
          hessian_m(j,i)=hessian_m(i,j)
       enddo
    enddo
  end subroutine hessian_mass_weighting

  !************************************************************************

  subroutine write_tm_force(force)
    !----------------------------------------------------------------
    !----------------------------------------------------------------
    implicit none

    real(double),intent(in) :: force(:,:)
    integer :: i,j,nvar
    nvar=ubound(force,1)

    open(unit=45,file="force",status='unknown')
    write(45,'(''$forceapprox          (8f10.5)'')')
    write(45,'(''# numerical force constant matrix for project:'')')  
    write(45,'(''# --->     <---'')')
    do i=1, nvar
       write(45,'((8f10.5))')(force(i,j),j=1,i)
       if(mod(i,8).eq.0 .and. i .ne. nvar) &
            & write(45,'(a15,i5,a3,i5)')'#  coordinates',i+1,' to',min(i+8,nvar)
    enddo
    write(45,'(''$end'')') 
    close (45)
  end subroutine write_tm_force

  !************************************************************************


  subroutine calc_atom_force(force,unit)
    !----------------------------------------------------------------
    ! still not working
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    use snf_molecule
    implicit none
    real(double),intent(in) :: force(:,:)
    integer,optional :: unit
    integer :: i,j,nvar,out
    nvar=ubound(force,1)
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,'(a50)')'writing force constant matrix'
    write(out,'(a50)')'-----------------------------'
    do i=1, nvar,3
       write(out,'((8f10.5))')((force(i,j)+force(i+1,j+1)+force(i+2,j+2))&
            & ,j=1,i,3)
    enddo
  end subroutine calc_atom_force

  !************************************************************************

  subroutine print_force_matrix(force,filename)
    use snf_param
    !----------------------------------------------------------------
    !     output routine (may be not working)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000
    !----------------------------------------------------------------
    implicit none
    real(double),intent(in) :: force(:,:)
    character*(*) filename
    integer :: i,j,nvar
    real(double) :: fac
    fac = 1.d0/bohr2ang
    nvar=ubound(force,1)
    open(unit=45,file=filename,status='unknown')
    write(45,*) &
         & 'hessian matrix in cartesian coordinates'
    write(45,*)'nvar=',nvar    
    write(45,'(4d18.10)')((force(i,j)*fac,j=1,i),i=1,nvar)
    close (45)
  end subroutine print_force_matrix


  !************************************************************************

end module snf_force

