
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_irred_atoms
  use f90_kind
  use snf_error

  private  
  public :: create_irred_atoms,destroy_irred_atoms,print_irred_atoms
  public :: check_geometry,get_point_grp,is_point_grp

  !************************************************************************

contains

  function is_point_grp(molecule,threshold,equal,pointgrp) result(valid)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_tools,only : getint
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double),intent(in) :: threshold
    character(len=3),intent(in) :: pointgrp
    character(len=132) :: errbuff
    logical :: equal,valid
    integer :: order
    character(len=7) :: symmel
    external equal
    if(pointgrp.eq.'c1 ') then
       valid=.true.
    else if(pointgrp.eq. 'ci ') then
       valid=check_symel(molecule,'sz2_1  ',threshold,equal)
    else if(pointgrp.eq. 'cs ') then
       valid=check_symel(molecule,'sh     ',threshold,equal)
    else if(pointgrp(1:1).eq. 's') then
       order=getint(pointgrp,2)
       write(symmel,'(''sz'',I1,''_1'')')order
       valid=check_symel(molecule,symmel,threshold,equal)
    else if(index('cd',pointgrp(1:1)) .ne.0) then
       order=getint(pointgrp,2)
       write(symmel,'(''cz'',I1,''_1'')')order
       if(check_symel(molecule,symmel,threshold,equal)) then
          if(pointgrp(3:3).eq. 'h') then
             valid=check_symel(molecule,'sh     ',threshold,equal)
          else if(pointgrp(3:3).eq. 'v') then
             valid=check_symel(molecule,'sv2_0  ',threshold,equal)
          else if(pointgrp(3:3).eq. 'd') then
             if(2*order.le.9) then
                write(symmel,'(''sd'',I1,''_0  '')')2*order
             else
                write(symmel,'(''sd'',I2,''_0 '')')2*order
             endif
             valid=check_symel(molecule,symmel,threshold,equal)
          else
             valid=.true.
          endif
          if(pointgrp(1:1).eq. 'd') then
             valid = valid .and. check_symel(molecule,'sv2_0  ',threshold,equal)
          endif
       else
          valid=.false.
       endif
    else
       if(pointgrp(1:1).eq.'o') then
          valid=check_symel(molecule,'o1402  ',threshold,equal)
          valid=valid .and. check_symel(molecule,'o1401  ',threshold,equal)
          if(pointgrp.eq.'oh') then
             valid=valid .and. check_symel(molecule,'sh     ',threshold,equal)
          endif
       else if(pointgrp(1:1).eq.'i') then
          valid=check_symel(molecule,'i1502  ',threshold,equal)
          valid=valid .and.check_symel(molecule,'i1501  ',threshold,equal)
          if(pointgrp.eq.'ih') then
             valid=valid .and. check_symel(molecule,'sz2_1  ',threshold,equal)
          endif
       else if(pointgrp(1:1).eq. 't') then
          valid=check_symel(molecule,'o1301  ',threshold,equal)
          valid=valid.and.check_symel(molecule,'o1303  ',threshold,equal)
          if(pointgrp.eq.'td') then
             valid=valid .and.check_symel(molecule,'p1401  ',threshold,equal)
          endif
          if(pointgrp.eq.'th') then
             valid=valid .and.check_symel(molecule,'sh     ',threshold,equal)
          endif
       else
          write(errbuff,*) 'Unknown point-group: ', pointgrp
          call serror('is_point_grp',2,errbuff)
       endif
    end if
  end function is_point_grp

  subroutine get_point_grp(molecule,threshold,equal,pointgrp)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double),intent(in) :: threshold
    character(len=3),intent(out) :: pointgrp
    logical :: equal
    logical :: is_sh
    integer :: order
    character(len=7) :: symmel
    external  equal
    order=8
    pointgrp='   '
    is_sh = check_symel(molecule,'sh     ',threshold,equal)
    cnz: do 
       write(symmel,'(''cz'',I1,''_1  '')')order
       if (check_symel(molecule,symmel,threshold,equal)) exit cnz
       order=order-1
    end do cnz
    if(order.eq.5 .and. check_symel(molecule,'i1502  ',threshold,equal)) then
       if(check_symel(molecule,'sz2_1  ',threshold,equal)) then 
          pointgrp='ih'
       else
          pointgrp='i'
       endif
    else if(order.eq.4 .and. check_symel(molecule,'o1402  ',threshold,equal)) then
       if(is_sh) then 
          pointgrp='oh'
       else
          pointgrp='o'
       endif
    else if(order.eq.2 .and. check_symel(molecule,'o1301  ',threshold,equal)) then
       if(is_sh) then 
          pointgrp='th'
       else if(check_symel(molecule,'p1401  ',threshold,equal)) then
          pointgrp='td'
       else
          pointgrp='t'
       endif
    else if(order.ge. 2) then     ! Cnx, Dnx ...
       if(check_symel(molecule,'cx2_0  ',threshold,equal)) then
          ! --> Dn, Dnh, Dnd
          write(pointgrp,'(''d'',I1)')order
          if(2*order.le.9) then
             write(symmel,'(''sd'',I1,''_0  '')')2*order
          else
             write(symmel,'(''sd'',I2,''_0 '')')2*order
          endif
          if(is_sh) then 
             pointgrp(3:3)='h'
          else if(check_symel(molecule,symmel,threshold,equal)) then
             pointgrp(3:3)='d'
          endif
       else                  ! --> Cn, Cnh, Cnv
          write(pointgrp,'(''c'',I1)')order
          if(2*order.le.9) then
             write(symmel,'(''sz'',I1,''_1  '')')2*order
          else
             write(symmel,'(''sz'',I2,''_1 '')')2*order
          endif
!JN          write(*,*)'symmel : ',symmel
          if(is_sh) then
             pointgrp(3:3)='h'
          else if(check_symel(molecule,'sv2_0  ',threshold,equal)) then
             pointgrp(3:3)='v'
          else if(check_symel(molecule,symmel,threshold,equal)) then
             if(2*order.le.9) then
                write(pointgrp,'(''s'',I1,1X)')2*order
             else
                write(pointgrp,'(''s'',I2)')2*order
             endif
          endif
       endif
    else                     ! --> Cs, Ci, C1
       if(is_sh) then
          pointgrp='cs'
       else if(check_symel(molecule,'sz2_1  ',threshold,equal)) then
          pointgrp='ci'
       else 
          pointgrp='c1'
       endif
    endif
  end subroutine get_point_grp

  !************************************************************************

  function check_symel(molecule,symmel,threshold,equal) result(valid)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_symmat,only:symel_mat
    implicit none
    logical valid
    type(type_atom),intent(in) :: molecule(:)
    character(LEN=7),intent(in) :: symmel
    real(double),intent(in) :: threshold
    type(type_atom) :: rot_at
    integer :: atom1_ind,atom2_ind,natoms
    logical :: equal
    real(double) :: symmat(3,3)
    external  equal
    natoms = ubound(molecule,1)
    symmat=symel_mat(symmel)
    atom1: do atom1_ind=1,natoms
       !----------------------------------------------------------------
       ! transform atom1 with all symmetry elements of the point group
       !----------------------------------------------------------------
       rot_at%element=molecule(atom1_ind)%element
       rot_at%mass=molecule(atom1_ind)%mass
       rot_at%cartcoord=matmul(symmat,molecule(atom1_ind)%cartcoord)
       !-------------------------------------------------------------
       ! compare coordinates of transformed atom1 and all other atoms
       !-------------------------------------------------------------
       atom2_ind = 1
       atom2: do while (.not.equal(rot_at,molecule(atom2_ind),threshold))
          atom2_ind = atom2_ind+1
          if (atom2_ind > natoms) exit atom2
       enddo atom2
       if(atom2_ind > natoms) then
          valid=.false.
          return
       end if
    end do atom1
    valid = .true.
  end function check_symel

  subroutine check_geometry(molecule,char_table,equal,unit)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom
    use snf_char_table,only: type_char_table

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    integer,optional :: unit

    integer :: out,i
    logical :: isfine,found!,mcheck
    real(double) :: threshold,max_thres
    real(double),pointer :: thresholds(:)
    Logical :: equal
    external equal

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    allocate(thresholds(char_table%nsymel))
    isfine=.true.
    found = .false.
    max_thres=0.D0
    sym: do i=2,char_table%nsymel
       threshold = 1.0d-16
       thres: do while(threshold .lt. 1.d-3)
          found = check_symel(molecule,char_table%symel(i),threshold,equal)
          if (found) exit thres
          threshold = threshold * 10.d0
       end do thres
       if(found) then
          thresholds(i)=threshold
          max_thres=max(max_thres,threshold)
       else
          thresholds(i) = -1.d0
       end if
       isfine=isfine .and. found
    end do sym

    if(isfine) then
       write(out,*)
       write(out,'(''   Geometry fits to '',A3,'' symmetry within '',G9.2,'' bohr.'')')Char_table%point_group,max_thres
    else
       write(out,*)' *************************************************'
       write(out,*)' ERROR: Molecule has not desired symmetry: ',Char_table%point_group
       write(out,*)' *************************************************'
       sym1: do i=2,char_table%nsymel
          if(thresholds(i).lt. 0) then
             write(out,'(a,a,a)')'  Symmetry operation ',char_table%symel(i),' failed.'
          else
             write(*,*)' Symmetry operation ',char_table%symel(i),' within:',thresholds(i)
          endif
       end do sym1
    endif

    deallocate(thresholds)
  end subroutine check_geometry

  subroutine create_irred_atoms(molecule,char_table,atom_index,&
     & symm_index,irred_atom_index,nirred_atoms)
    !----------------------------------------------------------------
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    use snf_molecule,only:type_atom,atom_equal
    use snf_symmat,only:symel_mat
    use snf_char_table,only: type_char_table

    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_char_table),intent(in) :: char_table
    type(type_atom) :: rot_at
    integer,pointer :: atom_index(:),symm_index(:),irred_atom_index(:)
    integer i,atom1_ind,atom2_ind
    integer natoms,nirred_atoms
    real(double) :: threshold
    logical :: isfine
    !----------------------------------------------------------------
    ! determination of non-redundant atoms ("irreducible atoms")
    !----------------------------------------------------------------
    natoms = ubound(molecule,1)
    allocate(atom_index(natoms),&
         & symm_index(natoms),&
         & irred_atom_index(natoms))
    atom_index(:) = -1
    nirred_atoms = 0
    do atom1_ind=1,natoms
       if(atom_index(atom1_ind) .eq. -1) then
          !-------------------------------------------------------------
          ! neglect dummy atoms
          !-------------------------------------------------------------
          if(molecule(atom1_ind)%element.eq.'Q') cycle
          nirred_atoms=nirred_atoms+1
          irred_atom_index(nirred_atoms)=atom1_ind
          do i=1,char_table%nsymel
             !----------------------------------------------------------------
             ! transform atom1 with all symmetry elements of the point group
             !----------------------------------------------------------------
             isfine = .false.
             threshold = 1.0d-16
             do while(.not. isfine .and. threshold .lt. 1.d-3)
                atom2_ind = 1
                rot_at%element=molecule(atom1_ind)%element
                rot_at%mass=molecule(atom1_ind)%mass
                rot_at%cartcoord=matmul(symel_mat(char_table%symel(i)), &
                     molecule(atom1_ind)%cartcoord)
                !-------------------------------------------------------------
                ! compare coordinates of transformed atom1 and all other atoms
                !-------------------------------------------------------------
                atom2_ind = 1
                atom2: do while (.not. atom_equal(rot_at,molecule(atom2_ind),threshold))
                   atom2_ind = atom2_ind+1
                   if (atom2_ind > natoms) exit atom2
                enddo atom2
                if(atom2_ind > natoms) then
                   isfine = .false.
                   threshold = threshold * 10.d0
                else 
                   isfine=.true.
                endif
             enddo
             if(.not. isfine) then
                write(*,*)' *************************************************'
                write(*,*)' ERROR: Molecule has not desired symmetry.'
                write(*,*)' Could definitly not map symmetry equivalent atoms'
                write(*,*)' Failed Symmetry element: ',char_table%symel(i)
                write(*,*)' *************************************************'
                stop
             endif
             !----------------------------------------------------------------
             ! assign symmetry equivalent atom to irred. atom and 
             ! correspondig symmetry operation if this atom has not been 
             ! assigned to another atom yet
             !----------------------------------------------------------------
             if(atom_index(atom2_ind) .eq. -1) then
                atom_index(atom2_ind) = atom1_ind
                symm_index(atom2_ind) = i
             endif
          enddo
       endif
    enddo
  end subroutine create_irred_atoms

  !************************************************************************

  subroutine print_irred_atoms(atom_index,symm_index,irred_atom_index,nirred_atoms,unit)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    ! modified by Johannes Neugebauer, Uni Erlangen, 05/2001
    !----------------------------------------------------------------
    implicit none
    integer,optional :: unit
    integer,intent(in) :: atom_index(:),symm_index(:),irred_atom_index(:)
    integer :: nirred_atoms,i,out

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*)
    write(out,'('' Information about symmetry redundant atoms'')')
    write(out,'('' ----------------------------------------------'')')
    write(out,'(''   Number of non-redundant atoms:'',i3)') nirred_atoms
    write(out,*)
    write(out,'(''   List of non-redundant atoms:'')')
    write(out,'(3X,16i4)')(irred_atom_index(i),i=1,nirred_atoms)
    write(out,*)
    write(out,'(''   atom No. | related non-redundant atom | symmetry op.'')')
    write(out,'(''   ---------|----------------------------|-------------'')')
    do i=1,ubound(atom_index,1)
       write(out,'(3X,i7,a3,i26,a3,i11)') i,'  |',atom_index(i),'  |',symm_index(i)
    enddo
    write(out,*)   
  end subroutine print_irred_atoms

  !************************************************************************

  subroutine destroy_irred_atoms(atom_index,symm_index,irred_atom_index)
    !----------------------------------------------------------------
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    integer,pointer :: atom_index(:),symm_index(:),irred_atom_index(:)

    if (associated(atom_index)) deallocate(atom_index)
    if (associated(symm_index)) deallocate(symm_index)
    if (associated(irred_atom_index)) deallocate(irred_atom_index)
  end subroutine destroy_irred_atoms

  !************************************************************************

end module snf_irred_atoms
