
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_mopac
  use f90_kind
  use snf_error

  private
  public :: print_mopac_frequencies,print_mopac_molecule,print_mopac_norm_modes
  public :: print_mopac_zmat

contains

  !************************************************************************

  subroutine print_mopac_frequencies(molecule,spectrum,xmolfl,mopac_file,unit)
    !----------------------------------------------------------------
    ! print out mopac fake output
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    ! extended by J. Neugebauer, Uni Erlangen, 06/2001
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_spectrum, only : type_peak
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    type(type_peak),intent(in) :: spectrum(:)
    character(len=*),intent(in) :: mopac_file
    integer,optional,intent(in) :: unit
    integer :: mop_unit,out
    logical,intent(in) :: xmolfl
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    mop_unit=55
    open(unit=mop_unit,file=mopac_file,status='unknown')
    write (mop_unit,*) ' MOPAC:  VERSION fudged MOPAC output for molden ...'
    if(xmolfl) call print_mopac_zmat(molecule,unit=mop_unit)
    call print_mopac_molecule(molecule,unit=mop_unit)
    write(mop_unit,*)
    write(mop_unit,'(a)') '           MASS-WEIGHTED COORDINATE ANALYSIS'
    call print_mopac_norm_modes(spectrum,unit=mop_unit)
    write(mop_unit,*)
    close(mop_unit)
    write(out,'(''   Created mopac fake output file : '',A)') mopac_file
  end subroutine print_mopac_frequencies

  !************************************************************************

  subroutine print_mopac_molecule(molecule,unit)
    !----------------------------------------------------------------
    ! print out cartesian coordinates for mopac for fake output
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_param
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    integer,optional ::unit
    integer         :: i,natoms,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    natoms= ubound(molecule,1)
    write (out,*) ' '
    write (out,*) '        ORIENTATION OF MOLECULE IN FORCE CALCULATION'
    write (out,*) ' '
    write (out,*) '   NO.       ATOM         X         Y         Z'
    write (out,*) ' '
    do i=1,natoms
       write(out,110) i,&
            molecule(i)%nucl_charge,&
            molecule(i)%cartcoord(1)*bohr2ang,&
            molecule(i)%cartcoord(2)*bohr2ang,&
            molecule(i)%cartcoord(3)*bohr2ang
    enddo
110 format(i6,i10,4x,3f10.4)
  end subroutine print_mopac_molecule

  !************************************************************************

  subroutine print_mopac_zmat(molecule,unit)
    !----------------------------------------------------------------
    ! print out mopac Z-matrix
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 06/2001
    !
    ! this part of the output is necessary if xmol should be
    ! applied. it is not necessary for molden.
    !----------------------------------------------------------------
    use snf_molecule, only : type_atom
    use snf_param
    implicit none
    type(type_atom),intent(in) :: molecule(:)
    real(double) :: datz(3)
    character(len=78),pointer :: datline(:)
    character(len=132):: buffer
    character(len=80) :: mofile
    character(len=3) :: count_string
    character(len=2) :: elsym,dum
    integer,optional :: unit
    integer         :: i,out,info,system,counter,j
    integer         :: dati(3),datc(3)
    logical :: ex
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write (out,'(a)') 
    write (out,'(a)') '    ATOM   CHEMICAL  BOND LENGTH    BOND ANGLE     TWIST ANGLE'
    write (out,'(a)') '   NUMBER  SYMBOL    (ANGSTROMS)     (DEGREES)      (DEGREES)'
    write (out,'(a)') '    (I)                  NA:I          NB:NA:I      NC:NB:NA:I&
      &     NA   NB   NC'
    write (out,*) 
    !----------------------------------------------------------------
    ! based will be used to generate z-matrix
    !----------------------------------------------------------------
    counter=0
    mofile='FOR005'
    do
       inquire (file=mofile,exist=ex)
       if(.not. ex) exit
       counter=counter+1
       write(count_string,'(i3)') counter
       mofile= 'FOR005' //'.' //trim(adjustl(count_string))
    enddo
    inquire(file='bas.bat',exist=ex)
    if(ex) call serror('print_mopac_zmat',2,'File bas.bat already exists!')
    open(unit=20 ,file='bas.bat',status='UNKNOWN')
    write(20,'(a)') 'based << !'
    write(20,'(a)') 're coord tm'
    write(20,'(3a)') 'wr ',trim(adjustl(mofile)),' mop'
    write(20,'(a)') 'x'
    write(20,'(a)') '!'
    close(20)
    info=system('chmod u+x bas.bat')
    if(info.ne.0) call serror('print_mopac_zmat',2,'Could generate bas.bat',out)
    call serror('print_mopac_zmat',-1,'Calling based to generate mopac z-matrix.')
    info=system('bas.bat')
    if(info.ne.0) call serror('print_mopac_zmat',2,'Could not execute based',out)
    call serror('print_mopac_zmat',-1,'Mopac z-matrix completed.')
    info=system('rm bas.bat')
    if(info.ne.0) call serror('print_mopac_zmat',2,'Error while removing batch &
      &file',out)
    open(20, file=trim(adjustl(mofile)))
    counter=0
    do
      read(20,'(a)',end=100) buffer
      counter=counter+1
      cycle
100   exit
    end do
    if(counter.eq.0) call serror('print_mopac_zmat',2,'Empty mopac input file!')
    allocate(datline(counter))
    !----------------------------------------------------------------
    ! the first three lines contain no necessary data
    !----------------------------------------------------------------
    rewind(20)
    do i=1,3
      read(20,'(a)') buffer
    end do
    do i=4,counter
      read(20,'(a)') datline(i)
      read(datline(i),fmt=110) elsym,dum,(datz(j),dati(j),j=1,3),&
       &dum,(datc(j),j=1,3)
      if(i.gt.6) then
        write(datline(i),fmt=120)i-3,elsym,(datz(j),'  *',j=1,3),&
         &(datc(j),j=1,3)
      else if(i.eq.4) then
        write(datline(i),fmt=130)i-3,elsym
      else if(i.eq.5) then
        write(datline(i),fmt=140)i-3,elsym,datz(1),'  *',datc(1)
      else if(i.eq.6) then
        write(datline(i),fmt=150)i-3,elsym,(datz(j),'  *',j=1,2),&
         &datc(1),datc(2)
      end if
      write(out,'(a)') datline(i)
    end do
    close(20)
    info=system('rm '//trim(adjustl(mofile)))
    if(info.ne.0) call serror('print_mopac_zmat',2,'Error while removing temporary&
      & mopac file')
    info=system('rm fort.41')
    if(info.ne.0) call serror('print_mopac_zmat',2,'Error while removing &
     &file fort.41')
    info=system('rm TMP.LOG')
    if(info.ne.0) call serror('print_mopac_zmat',2,'Error while removing &
     &file TMP.LOG')

110  format(a2,a2,3(f14.8,i4),a2,3i4)
120  format(i7,6x,a2,1x,2(f14.5,a3),f11.5,a3,i4,2i5)
130  format(i7,6x,a2)
140  format(i7,6x,a2,1x,f14.5,a3,31x,i4)
150  format(i7,6x,a2,1x,2(f14.5,a3),14x,i4,i5)
    deallocate(datline)
  end subroutine print_mopac_zmat

  !************************************************************************


  subroutine  print_mopac_norm_modes(spectrum,unit)
    !----------------------------------------------------------------
    ! matout prints a rectangular matrix of eigenvectors and eigenvalues
    !----------------------------------------------------------------
    use snf_spectrum, only : type_peak
    use snf_param
    implicit none
    type(type_peak),intent(in) :: spectrum(:)
    integer,optional :: unit
    integer ka,kb,kc,la,lb,lc,nr,nc, i,j,out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    nc=ubound(spectrum,1)
    nr=ubound(spectrum(1)%norm_mode,1)

    ka=1
    kc=6
60  kb=min0(kc,nc)
    write (out,100) (i,i=ka,kb)
    if(abs(spectrum(1)%freq) .ge.1.d-10) &
         &   write (out,110) (spectrum(i)%freq,i=ka,kb)
    write (out,120)
    la=1
    lc=40
70  lb=min0(lc,nr)
    do  i=la,lb
       !       if(itext(i).eq.' s')write(out,120)
       write (out,130) i,(spectrum(j)%norm_mode(i),j=ka,kb)
       !       write (out,130) i,(spectrum(j)%norm_mode(i)*bohr2ang,j=ka,kb)
    enddo
    if (lb.eq.nr) go to 90
    la=lc+1
    lc=lc+40
    write (out,140)
    go to 70
90  if (kb.eq.nc) then
       return
    endif
    ka=kc+1
    kc=kc+6
    if (nr.gt.25) write (out,140)
    go to 60

100 format (////,3x,9h ROOT NO.,i5,9i12)
110 format (/8x,10f12.5)
120 format (2h  )
130 format (6x,i4,f10.5,10f12.5)
140 format (1h1)

  end subroutine print_mopac_norm_modes

  !************************************************************************

end module snf_mopac
