
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_param
  USE f90_kind
  !----------------------------------------------------------------
  ! constants module for use in all other modules
  ! values from http://physics.nist.gov/ 
  ! and P.J. Mohr, B.N. Taylor: CODATA recommended values of the
  ! fundamental physical constants,
  ! Rev. Mod. Phys. 72 (2000), No.2, 351-495.
  !
  ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 2000
  ! modified by Johannes Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  real(double), parameter :: pi=3.1415926535897932384626433832795029d0
  real(double), parameter :: speedoflight=2.99792458d08
  real(double),  parameter :: avogadro=6.02214199d23
  real(double),  parameter :: planck_h=6.62606876d-34
  real(double),  parameter :: el_charge=1.602176462d-19
  real(double),  parameter :: el_mass=9.10938188d-31
  real(double),  parameter :: mu_0=4*pi*1.d-07
  real(double),  parameter :: planck_hquer=planck_h/(2*pi)            ! 1.054571596d-34
  real(double),  parameter :: atomic_mass_unit=1/avogadro*1d-03       ! 1.66056d-27
  real(double),  parameter :: epsilon_0=1/(speedoflight*speedoflight*mu_0) ! 8.85419d-12
  real(double),  parameter :: bohr=4.0d0*pi*epsilon_0*planck_hquer*planck_hquer/(el_mass*el_charge*el_charge) ! 5.291772083d-11
  real(double),  parameter :: hartree=el_charge**4*el_mass/((4*pi*epsilon_0)**2*planck_hquer**2) !  4.35974381d-18 
  real(double),  parameter :: bohr2ang=bohr*1.0d10      ! 0.529177260d0
  real(double),  parameter :: hartree2kj_mol=hartree*avogadro*1.d-03 !   2.62549992d03
  real(double),  parameter :: esu= 1.0d-01/speedoflight  ! 3.33564d-10
  real(double),  parameter :: debey=esu*1.d-2*1.0d-18   ! 3.33564d-30
  real(double),  parameter :: kjmol2hartree=0.0003809d0
  real(double),  parameter :: boltz=1.3806503d-23     ! in j/k
  real(double),  parameter :: invfinestruc=137.03599976d0 ! inverse fine-structure constant
  real(double),  parameter :: nm2freq=2.d0*pi*invfinestruc/10.0d0*bohr2ang
                          ! wavelength in [nm] to angular frequency in [a.u.]
  real(double),  parameter :: gasconstant=boltz*avogadro

end module snf_param
