
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_projection
  use f90_kind

  private
  public :: create_proj_op,print_proj_op,destroy_proj_op

  !************************************************************************

contains

  !************************************************************************

  subroutine create_proj_op(representation,char_table,proj_op)
    !----------------------------------------------------------------
    ! this routine creates projection operators of reducible 
    ! representations and irreducible representations from the character table
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    implicit none

    type(type_char_table),intent(in) :: char_table
    real(double) ,intent(in) ::representation(:,:,:)
    real(double), pointer :: proj_op(:,:,:)
    integer i,j
    double precision :: fac1,fac2
    allocate(proj_op(ubound(representation,1),ubound(representation,2),char_table%nirrep))

    proj_op(:,:,:) = 0.0
    do i=1,char_table%nirrep
       do j=1,char_table%nsymel
          fac1=char_table%l(j,i)
          proj_op(:,:,i)=proj_op(:,:,i)+fac1*representation(:,:,j)
       enddo
       fac2=dble(char_table%g(i))/dble(char_table%ordnung(i))
       proj_op(:,:,i)=fac2*proj_op(:,:,i)
    enddo
  end subroutine create_proj_op

  !*************************************************************************

  subroutine print_proj_op(proj_op,char_table,unit)
    !----------------------------------------------------------------
    ! output of representions of projection operators
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    use snf_char_table,only:type_char_table
    use snf_tools,only:printmat
    implicit none

    real(double),intent(in) ::  proj_op(:,:,:)
    integer,optional :: unit
    integer i,out
    type(type_char_table),intent(in) :: char_table

    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    write(out,*) 
    write(out,*)'projection operators of pointgroup:',char_table%point_group
    write(out,*)'--------------------------------------'
    do i=1,ubound(proj_op,3)
       write(out,*) 
       call printmat(proj_op(:,:,i),unit=out,text=char_table%mulliken(i),fmt='F8.3')
    enddo
  end subroutine print_proj_op

  !************************************************************************

  subroutine destroy_proj_op(proj_op)
    !----------------------------------------------------------------
    ! Destructor
    !
    ! written by Carsten Kind, Uni Koeln, 1994
    !----------------------------------------------------------------
    implicit none
    real(double),pointer ::  proj_op(:,:,:)

    deallocate(proj_op)
  end subroutine destroy_proj_op

  !************************************************************************

end module snf_projection

