
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_tools
  !------------------------------------------------------------------------
  ! written by C. Kind
  ! modified by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
  !----------------------------Typvereinbarung-----------------------------
  use f90_kind

  private
  public :: chr,getint,string2real, string2integer,test_uni_mat
  public :: inverse_matrix, gram_schmidt_procedure,clean,rmfile
  public :: testfile,exscript,prog_type,getwd,bfile

  PUBLIC :: PrintMat,ReadDMat,PrintVec,ReadVec
  INTERFACE ReadVec ! Reads Vector
     MODULE PROCEDURE ReadDVec,ReadIVec 
  END INTERFACE
  INTERFACE PrintMat ! Prints Matrix
     MODULE PROCEDURE PrintDMat,PrintIMat
  END INTERFACE
  INTERFACE PrintVec ! Prints Vector
     MODULE PROCEDURE PrintDVec,PrintIVec
  END INTERFACE
  !************************************************************************

contains

  !************************************************************************

  function chr(zahl)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    integer zahl,i1,i10,i100,i0
    character(len=3) :: chr
    i1=mod(zahl,10)
    i10=(mod(zahl,100)-i1)/10
    i100=(zahl-10*i10-i1)/100
    i0=ichar('0')
    if (i100 .eq. 0) then
       if (i10 .eq. 0) then
          if (i1 .eq. 0) then
             chr = '   '
          else
             chr =  '  '//char(i1+i0)
          endif
       else
          chr = ' '//char(i10+i0)//char(i1+i0)
       endif
    else
       chr=char(i100+i0)//char(i10+i0)//char(i1+i0)
    endif
  end function chr

  !**************************************************************************

  function string2integer(row)
    !------------------------------------------------------------------------
    ! written by C. Kind
    ! modified by J. Neugebauer, Universitaet Erlangen-Nuernberg, 05/2001
    !----------------------------Typvereinbarung-----------------------------
    implicit none
    character(len=*),intent(in):: row
    integer :: zahl,string2integer
    integer                   :: i,j,k
    !----------------------------Ausfuehrungsteil----------------------------
    k=0
    zahl=0
    i=len(row)
    do while(index('0123456789',row(i:i)) .EQ.0.and.i.ge.2)
       i=i-1
    enddo
    j=index('0123456789',row(i:i))
    do while(j.ne.0)
       zahl=zahl+nint((j-1)*10.d0**k)
       k=k+1
       i=i-1
       j=index('0123456789',row(i:i))
    end do
    if(index('-',row(i:i)).EQ.1) zahl=-zahl
    string2integer=zahl
  end function string2integer

  !******************************************************************************
  function string2real(row)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    !----------------------------Typvereinbarung-----------------------------
    implicit none
    character(len=*),intent(in):: row
    real(double) :: zahl,string2real
    integer      :: i,j,k,pos,upper,minus
    !----------------------------Ausfuehrungsteil----------------------------
    k=0
    zahl=0
    upper=len(trim(row))
    pos=index(row,'.')
    if (pos.EQ.0) pos=upper+1
    i=pos-1
    j=index('0123456789',row(i:i))
    do while(j.ne.0)
       zahl=zahl+(j-1)*10.d0**k
       k=k+1
       i=i-1
       j=index('0123456789',row(i:i))
    end do
    minus=i
    k=-1
    do i=pos+1,upper,1
       j=index('0123456789',row(i:i))
       if (j.ne.0) then
          zahl=zahl+(j-1)*10.d0**(k)
          k=k-1
       end if
    end do
    if(index('-',row(minus:minus)).EQ.1) zahl=-zahl
    string2real=zahl
  end function string2real

  !******************************************************************************

  function getint(string,start)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    character string*(*)
    integer start,pos,getint
    pos=start
    do while ((index('+-0123456789',string(pos:pos)) .ne. 0 ) .and. &
         & ((len(string)) .ge. pos))
       pos=pos+1
    enddo
    if (start .eq. pos) then
       getint=0
    else
       if (start-pos.eq.-6) then
          read(string(start:pos-1),12,err=10) getint
       elseif (start-pos.eq.-5) then
          read(string(start:pos-1),13,err=10) getint
       elseif (start-pos.eq.-4) then
          read(string(start:pos-1),14,err=10) getint
       elseif (start-pos.eq.-3) then
          read(string(start:pos-1),15,err=10) getint
       elseif (start-pos.eq.-2) then
          read(string(start:pos-1),16,err=10) getint
       elseif (start-pos.eq.-1) then
          read(string(start:pos-1),17,err=10) getint
       else
          stop 'error in getint'
       endif
    endif
    return

10  write(*,20) string(start:pos-1)
    write(*,30)
    getint=0
12  format(i6)
13  format(i5)
14  format(i4)
15  format(i3)
16  format(i2)
17  format(i1)
20  format(' error reading ',a,' in function getint!')
30  format(' function returns 0 (zero)!')
  end function getint


  !************************************************************************
  
  
  subroutine readivec(vector,unit,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    integer, dimension(:)            :: vector
    integer                          :: unit
    character(len=1)                 :: vectyp = 'd'
    character(len=*),optional :: fmt
    integer                          :: dev  ! device
    integer                          :: zeile  ! laufindizes
    integer                          :: anzzeilen
    character(len=*), parameter   :: indexform = '(4x,'
    character(len=6) :: vecelform
    dev = unit
    if(present(fmt)) then
       vecelform=fmt
    else
       vecelform='I4    '
    endif
    read(dev,*)
    read(dev,'("typ: ",a1,"   size of vec:",i4)')&
         vectyp,anzzeilen
    do zeile =1,anzzeilen
       read(dev,indexform//vecelform//')')vector(zeile)
    end do
    read(dev,*)
  end subroutine readivec


  !************************************************************************


  subroutine readdvec(vector,unit,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double), dimension(:)   :: vector
    integer                          :: unit
    character(len=1)                 :: vectyp = 'd'
    character(len=*),optional :: fmt
    integer                          :: dev  ! device
    integer                          :: zeile  ! laufindizes
    integer                          :: anzzeilen
    character(len=*), parameter   :: indexform = '(4x,'
    character(len=6) :: vecelform
    dev = unit
    if(present(fmt)) then
       vecelform=fmt
    else
       vecelform='e25.17'
    endif
    read(dev,*)
    read(dev,'("typ: ",a1,"   size of vec:",i4)')&
         vectyp,anzzeilen
    do zeile =1,anzzeilen
       read(dev,indexform//vecelform//')')vector(zeile)
    end do
    read(dev,*)
  end subroutine readdvec


  !************************************************************************


  subroutine printdvec(vector,unit,text,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double), dimension(:)   :: vector
    integer,optional                 :: unit
    character(len=*),optional        :: text
    character(len=*),optional :: fmt
    character(len=1),parameter       :: vectyp = 'd'
    integer                          :: dev  ! device
    integer                          :: zeile  ! laufindizes
    integer                          :: anzzeilen
    character(len=*), parameter   :: indexform = '(i3,'':'','
    character(len=6) :: vecelform
    if (present(unit)) then
       dev = unit
    else
       dev = 6
    end if
    if (present(text)) then
       write(dev,*)text
    else
       write(dev,*)
    end if
    if(present(fmt)) then
       vecelform=fmt
    else
       vecelform='e25.17'
    endif
    write(dev,'("typ: ",a1,"   size of vec:",i4)')&
         vectyp,ubound(vector,dim=1)
    anzzeilen   = ubound(vector,dim=1)
    do zeile =1,anzzeilen
       write(dev,indexform//vecelform//')')zeile,&
            vector(zeile)
    end do
    write(dev,*)
  end subroutine printdvec


  !************************************************************************


  subroutine printivec(vector,unit,text,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    integer, dimension(:)   :: vector
    integer,optional                 :: unit
    character(len=*),optional        :: text
    character(len=*),optional :: fmt
    character(len=1),parameter       :: vectyp = 'd'
    integer                          :: dev  ! device
    integer                          :: zeile  ! laufindizes
    integer                          :: anzzeilen
    character(len=*), parameter   :: indexform = '(i3,'':'','
    character(len=6) :: vecelform
    if (present(unit)) then
       dev = unit
    else
       dev = 6
    end if
    if (present(text)) then
       write(dev,*)text
    else
       write(dev,*)
    end if
    if(present(fmt)) then
       vecelform=fmt
    else
       vecelform='I4    '
    endif
    write(dev,'("typ: ",a1,"   size of vec:",i4)')&
         vectyp,ubound(vector,dim=1)
    anzzeilen   = ubound(vector,dim=1)
    do zeile =1,anzzeilen
       write(dev,indexform//vecelform//')')zeile,&
            vector(zeile)
    end do
    write(dev,*)
  end subroutine printivec


  !************************************************************************


  subroutine printdmat(matrix,unit,text,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double), dimension(:,:) :: matrix
    integer,optional                 :: unit
    character(len=*),optional        :: text,fmt
    character(len=1),parameter       :: mattyp = 'd'
    integer                          :: dev  ! device
    integer :: spalte, zeile  ! consecutive indices
    integer :: anfang, ende   ! variable boundaries for consecutive indices
    ! spalte
    integer :: anzspalten     ! number of columns, which can be printed in a row
    logical :: abbruch        ! controls the output 
    integer :: nrerstezeile,nrletztezeile,nrerstespalte,nrletztespalte

    character(len=*), parameter   :: indexform = '(i3,'':'','
    character(len=1),dimension(9),parameter :: multform &
         = (/'1','2','3','4','5','6','7','8','9'/)
    character(len=6) :: matelform
    if (present(unit)) then
       dev = unit
    else
       dev = 6
    end if
    if (present(text)) then
       write(dev,*)text
    else
       write(dev,*)
    end if
    if(present(fmt)) then
       matelform=fmt
    else
       matelform='e25.17'
    endif
    write(dev,'("typ: ",a1,"   size of dim=1:",i4,"   size of dim=2:",i4)')&
         mattyp,size(matrix,dim=1),size(matrix,dim=2)
    nrerstezeile   = lbound(matrix,dim=1)
    nrletztezeile  = ubound(matrix,dim=1)
    nrerstespalte  = lbound(matrix,dim=2)
    nrletztespalte = ubound(matrix,dim=2)

    anzspalten = 3
    abbruch=.false.
    anfang = nrerstespalte
    ende = nrerstespalte + anzspalten - 1
    do
       if (ende > nrletztespalte) then
          ende = nrletztespalte
          abbruch = .true.
       end if
       write(dev,'(4x,3(21x,i4))')(spalte,spalte=anfang,ende)
       do zeile = nrerstezeile,nrletztezeile
          write(dev,indexform//multform(anzspalten)//matelform//')')zeile,&
               (matrix(zeile,spalte),spalte=anfang,ende)
       end do
       if (ende == nrletztespalte) exit
       if (.not. abbruch) then
          anfang = anfang + anzspalten
          ende   = ende   + anzspalten
       else
          exit
       end if
       write(dev,*)
    end do
  end subroutine printdmat


  !************************************************************************


  subroutine printimat(matrix,unit,text,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    integer, dimension(:,:)          :: matrix
    integer,optional                 :: unit
    character(len=*),optional        :: text
    character(len=*),optional :: fmt
    character(len=1),parameter       :: mattyp = 'i'
    integer                          :: dev  ! device
    integer :: spalte, zeile  ! consecutive indices
    integer :: anfang, ende   ! variable boundaries for consecutive indices
    ! spalte
    integer :: anzspalten     ! number of columns, which can be printed in a row
    logical :: abbruch        ! controls the output
    integer :: nrerstezeile,nrletztezeile,nrerstespalte,nrletztespalte

    character(len=*), parameter   :: indexform = '(i4,'':'','
    character(len=1),dimension(9),parameter :: multform &
         = (/'1','2','3','4','5','6','7','8','9'/)
    character(len=6) :: matelform
    if (present(unit)) then
       dev = unit
    else
       dev = 6
    end if
    if (present(text)) then
       write(dev,*)text
    else
       write(dev,*)
    end if
    if(present(fmt)) then
       matelform=fmt
    else
       matelform='i4    '
    endif
    write(dev,'("typ: ",a1,"   size of dim=1:",i4,"   size of dim=2:",i4)')&
         mattyp,size(matrix,dim=1),size(matrix,dim=2)
    nrerstezeile   = lbound(matrix,dim=1)
    nrletztezeile  = ubound(matrix,dim=1)
    nrerstespalte  = lbound(matrix,dim=2)
    nrletztespalte = ubound(matrix,dim=2)

    anzspalten = 9
    abbruch=.false.
    anfang = nrerstespalte
    ende = nrerstespalte + anzspalten
    do
       if (ende > nrletztespalte) then
          ende = nrletztespalte
          abbruch = .true.
       end if
       write(dev,'(5x,9i4)')(spalte,spalte=anfang,ende)
       do zeile = nrerstezeile,nrletztezeile
          write(dev,indexform//multform(anzspalten)//matelform//')')zeile,&
               (matrix(zeile,spalte),spalte=anfang,ende)
       end do
       if (ende == nrletztespalte) exit
       if (.not. abbruch) then
          anfang = anfang + anzspalten
          ende   = ende   + anzspalten
       else
          exit
       end if
    end do
  end subroutine printimat


  !************************************************************************


  subroutine readdmat(matrix,unit,fmt)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double)         :: matrix(:,:)
    integer           :: unit
    character(len=1)     :: mattyp
    character(len=*),optional :: fmt
    integer      :: dev  ! device
    integer :: spalte, zeile  ! consecutive indices
    integer :: anfang, ende   ! variable boundaries for consecutive indices
    ! spalte
    integer :: anzspalten     ! number of columns, which can be printed in a row
    logical :: abbruch        ! controls the output
    integer :: nrerstezeile,nrletztezeile,nrerstespalte,nrletztespalte
    character(len=*), parameter   :: indexform = '(4x'
    character(len=1),dimension(9),parameter :: multform &
         = (/'1','2','3','4','5','6','7','8','9'/)
    character(len=6) :: matelform
    if(present(fmt)) then
       matelform=fmt
    else
       matelform='e25.17'
    endif
    dev = unit
    read(dev,*)
    read(dev,'(5x,a1,17x,i4,17x,i4)')&
         mattyp,nrletztezeile,nrletztespalte
    nrerstezeile   = 1
    nrerstespalte  = 1

    anzspalten = 3
    abbruch=.false.
    anfang = nrerstespalte
    ende = nrerstespalte + anzspalten - 1
    do
       if (ende > nrletztespalte) then
          ende = nrletztespalte
          abbruch = .true.
       end if
       read(dev,*)
       do zeile = nrerstezeile,nrletztezeile
          read(dev,indexform//multform(anzspalten)//matelform//')')&
               (matrix(zeile,spalte),spalte=anfang,ende)
       end do
       if (ende == nrletztespalte) exit
       if (.not. abbruch) then
          anfang = anfang + anzspalten
          ende   = ende   + anzspalten
       else
          exit
       end if
       read(dev,*)
    end do
  end subroutine readdmat

  !************************************************************************

  subroutine test_uni_mat(u)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double),intent(in) :: u(:,:)
    real(double), allocatable :: e2(:,:)
    real(double), allocatable :: orthotest2(:)
    integer :: i
    allocate(e2(ubound(u,2),ubound(u,2)))
    e2 = matmul(transpose(u),u)
    allocate(orthotest2(ubound(e2,1)))
    do i=1,ubound(e2,1)
       orthotest2(i)=dot_product(e2(:,i),e2(:,i))-e2(i,i)
    enddo
    write(*,*)'testergbnis der unitaeren matrix:'
    write(*,*)'---------------------------------'
    write(*,*)'einheitsmatrix:(',ubound(e2,1),',',ubound(e2,1),'):'
    do i=1,ubound(e2,1)
       write(*,*) 'normtest:',e2(i,i),'   orthotest:',orthotest2(i)
    enddo
    deallocate(e2,orthotest2)
  end subroutine test_uni_mat

  !************************************************************************

  subroutine inverse_matrix(a,b)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double),  intent(in)    :: a(:,:)
    real(double), intent(out) :: b(:,:)
    real(double),allocatable :: copy_a(:,:)
    integer,allocatable :: ipiv(:)
    integer :: dim,info,i
    dim=ubound(a,1)
    if(dim .ne. ubound(a,2)) &
         &  stop ' inverse_matrix: cannot invert non-quadratic matrix '
    allocate(copy_a(dim,dim),ipiv(dim))
    b=0.d0
    do i=1,dim
       b(i,i)=1.d0
    enddo
    copy_a=a
    call dgesv(dim, dim, copy_a, dim, ipiv, b, dim, info)

    deallocate(copy_a,ipiv)
    if (info .ne. 0) &
         &  stop 'error in inversion routine'
  end subroutine inverse_matrix

  !************************************************************************

  subroutine gram_schmidt_procedure(a,b,min_norm)
    !------------------------------------------------------------------------
    ! written by C. Kind
    !------------------------------------------------------------------------
    implicit none
    real(double),intent(in)  :: a(:,:),min_norm
    real(double),intent(out) :: b(:,:)
    real(double),allocatable ::  alpha(:,:),tmp(:),a2(:,:)
    integer :: max_j,dim_1,dim_2,i,k,l,ii
    real(double) :: max_norm,normfac,norms
    dim_1 = ubound(a,1)
    dim_2 = ubound(a,2)
    allocate(tmp(dim_1),alpha(dim_1,dim_1),a2(dim_1,dim_2))
    !----------------------------------------------------------------
    ! Gram-Schmidt-orthogonalization
    !----------------------------------------------------------------
    alpha = 0.d0
    a2 = a
    b = a
    do i=1,dim_2
       ! subtraction of last basisvector from all following basisvectors
       ! in preparation
       call dgemm('N','N',dim_1,dim_2-i+1,dim_1,-1.D0,alpha,dim_1,a2(1,i),dim_1,1.D0,b(1,i),dim_1)

       ! selection of next basis vector
       max_norm=dot_product(b(:,i),b(:,i))
       max_j=i
       do ii=i+1,dim_2
          norms=dot_product(b(:,ii),b(:,ii))
          if(norms .gt. max_norm) then
             max_norm = norms
             max_j=ii
          endif
       enddo
       ! sort new ith basisvector to the ith position
       if(i.ne.max_j) then
          tmp=b(:,i)
          b(:,i)=b(:,max_j)
          b(:,max_j)=tmp
          tmp=a2(:,i)
          a2(:,i)=a2(:,max_j)
          a2(:,max_j)=tmp
       endif
       ! set vector to zerovector when norm is too small
       if(max_norm .gt. min_norm) then
          normfac=1.d0/sqrt(max_norm)
          b(:,i)=normfac*b(:,i)
       else
          b(:,i)=0.d0
       endif
       ! generate new alpha-matrix
       do k=1,dim_1
          do l=1,k
             alpha(k,l)=b(k,i)*b(l,i)
             alpha(l,k)=alpha(k,l)
          enddo
       enddo
    enddo
    deallocate(alpha,tmp,a2)
  end subroutine gram_schmidt_procedure

  !************************************************************************

        subroutine clean(char)
!
!       written by Markus Reiher, University of Erlangen, Aug. 2001
!
        implicit none
        character(len=*) :: char
        integer          :: i
        intrinsic len
        do i=1,len(char)
          char(i:i)=' '
        enddo
        return
        end subroutine clean 

  !************************************************************************

      subroutine rmfile(rfile,mode)
!
!     if rfile exists, then remove it. if mode = 1, move file to file.bak
!
!     written by J. Neugebauer, University of Erlangen, 2001
      
      implicit none
      character(len=*) rfile
      character(len=132) nfile,mfile
      integer mode,info,system
      logical ex
      
      nfile=trim(adjustl(rfile))
      inquire(file=nfile,exist=ex)
      if(ex) then
        if(mode.eq.1) then
          mfile=trim(adjustl(rfile)) //'.bak'
          write(*,'(4a)') '** rmfile: moving file ',&
           &trim(adjustl(rfile)),' to ',trim(adjustl(mfile))
          info=system('mv '//nfile//' '//trim(mfile))
        else
          write(*,'(2a)') '** rmfile: removing file ',trim(adjustl(rfile))
          info=system('rm '//nfile)
        end if
        if(info.ne.0) then
          write(*,'(3a)') '** Error while (re)moving '//trim(adjustl(rfile))
        end if
      end if
      end subroutine rmfile

  !************************************************************************

      subroutine bfile(rfile)
!
!     if rfile exists, then create a write-protected backup rfile.bak 
!
!     written by J. Neugebauer, University of Erlangen, 2002
      
      implicit none
      character(len=*) rfile
      character(len=132) nfile,mfile
      integer info,system
      logical ex,bex
      
      nfile=trim(adjustl(rfile))
      inquire(file=nfile,exist=ex)
      if(ex) then
        inquire(file=trim(adjustl(rfile))//'.bak',exist=bex)
        mfile=trim(adjustl(rfile)) //'.bak'
        if(.not.bex) then
          write(*,'(4a)') '** bfile: creating backup ',&
           &trim(adjustl(mfile)),' of ',trim(adjustl(rfile))
          info=system('cp '//nfile//' '//trim(mfile))
          if(info.ne.0) then
            write(*,'(3a)') '** Error while creating'&
     &//trim(adjustl(rfile))
          end if
          info=system('chmod a-w '//trim(mfile))
          if(info.ne.0) then
            write(*,'(3a)') '** Error while creating write-protected &
     &file '//trim(adjustl(rfile))
          end if
        else
          write(*,'(5a)') '** bfile: ',trim(adjustl(mfile)),&
           & ' already exists'
        end if
      else
          write(*,'(5a)') '** bfile: ',trim(adjustl(rfile)),&
           & ' does not exist'
      end if
      end subroutine bfile

  !************************************************************************

      subroutine testfile(rfile)
!
!     if rfile does not exist, then stop.
!
!     written by J. Neugebauer, University of Erlangen, 2001
      
      implicit none
      character(len=*) rfile
      character(len=132) nfile
      logical ex
      
      nfile=trim(adjustl(rfile))
      inquire(file=nfile,exist=ex)
      if(.not.ex) then
        write(*,'(3a)') '** testfile: file '//trim(adjustl(rfile))//&
         &' does not exist.'
        stop
      end if
      end subroutine testfile

  !************************************************************************

      subroutine exscript(rfile)
!
!     make script rfile executable and execute it.
!
!     written by J. Neugebauer, University of Erlangen, 2001
      
      implicit none
      character(len=*) rfile
      character(len=132) nfile
      integer info,system
      
      call testfile(rfile)
      nfile=trim(adjustl(rfile))
      info=system('chmod u+x '//nfile)
      if(info.ne.0) goto 900
      write(*,'(2a)') '** exscript: executing '//trim(adjustl(rfile))
      write(*,'(72(''*''))') 
      info=system(nfile)
      if(info.ne.0) goto 900
      write(*,'(72(''*''))') 
      return
900   write(*,'(3a)') '** exscript: could not execute '//trim(adjustl(rfile))
      stop
      end subroutine exscript

  !************************************************************************

      subroutine prog_type(mode,prtype,catype)
!
!     reads mode value, returns progtype and calctype
!
!     written by J. Neugebauer, University of Erlangen, 2001
      
      implicit none
      integer maxmod
      integer,intent(in) :: mode 
      parameter(maxmod=30)
      character(len=10) prtype,catype,progtype(maxmod),calctype(maxmod)
      data progtype / &
    & 'TURBOMOLE ','TURBOMOLE ','TURBOMOLE ','TURBOMOLE ','TURBOMOLE ',&
    & 'TURBOMOLE ','TURBOMOLE ','TURBOMOLE ','          ','          ',&
    & '          ','          ','          ','          ','          ',&
    & '          ','          ','          ','          ','DALTON    ',&
    & 'DALTON    ','DALTON    ','DALTON    ','DALTON    ','DALTON    ',&
    & '          ','          ','          ','          ','          '/
    data calctype / &
    & 'DSCF      ','RIDFT     ','RIMP2     ','MPGRAD    ','DSCF/ESCFI',&
    & 'DSCF/ESCFA','RI/ESCFI  ','RI/ESCFA  ','          ','          ',&
    & '          ','          ','          ','          ','          ',&
    & '          ','          ','          ','          ','SCF       ',&
    & 'MP2       ','CC2       ','CCS       ','CCSD      ','MCSCF     ',&
    & '          ','          ','          ','          ','          '/
      if(mode.ge.1.and.mode.le.maxmod) then
        prtype=progtype(mode)
        catype=calctype(mode)
      else
        write(*,'(a)') '** progtype: unknown program type'
        stop
      end if
      if(prtype.eq.'          '.or.catype.eq.'          ') then
        write(*,'(a)') '** progtype: unknown program type'
        stop
      end if
       
      end subroutine prog_type

  !************************************************************************

      subroutine getwd(wd)
!
!     writes the working directory to "wd"
!
!     written by J. Neugebauer, University of Erlangen, 2002
    
      implicit none
      character(len=255) wd
      integer info, system
      info=system('pwd > pwd.tmp.tmp')
      if(info.ne.0) goto 900
      open(unit=25,file='pwd.tmp.tmp')
      read(25,*) wd
      close(25)
      call rmfile('pwd.tmp.tmp.',0)
900   write(*,'(3a)') '** getwd: could get working directory'
      stop
      end subroutine getwd

  !************************************************************************

end module snf_tools
