
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_xmol
  !---------------------------------------------------------------- 
  ! generate output for xmol (as "XYZ")
  ! NOTE: in order to avoid problems with the linewidth, you 
  !       should add the line:
  !
  !        1.000 setlinewidth
  !
  !       in every xmol output file after line:
  !
  !        %%EndSetup
  ! 
  ! written by J. Neugebauer, Uni Erlangen, 06/2001
  !---------------------------------------------------------------- 
  use f90_kind
  use snf_error

  private
  public :: print_xmol_norm

contains

 !************************************************************************  

  subroutine print_xmol_norm(molecule,spectrum,xmol_file,scale,unit)
  !---------------------------------------------------------------- 
  ! print normal modes readable in xmol (as "XYZ")
  ! 
  ! written by J. Neugebauer, Uni Erlangen, 06/2001
  ! 
  ! scale factors in snf.f90 were choosen according to the ones
  ! in mop2xvib.pl
  !---------------------------------------------------------------- 
  use snf_molecule, only : type_atom
  use snf_spectrum, only : type_peak
  use snf_param
  implicit none
  type(type_atom),intent(in) :: molecule(:)
  type(type_peak),intent(in) :: spectrum(:)
  character(len=*),intent(in) :: xmol_file 
  character(len=5) :: buff
  real(double),intent(in) :: scale
  integer,optional,intent(in) :: unit
  integer :: xmol_unit,out,nnorm,i,j,jj,natom
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    xmol_unit=55
    open(unit=xmol_unit,file=xmol_file,status='unknown')
    nnorm=ubound(spectrum,1)
    natom=ubound(molecule,1)
    buff='     '
    do i=1,nnorm
      write(buff,'(i5)') i
      write(xmol_unit,'(i5)') natom
      write(xmol_unit,'(3a,f11.5,a)')'* Frequency(',trim(adjustl(buff)),&
       &')  ',spectrum(i)%freq,' cm-1'
      do j=1,natom
        write(xmol_unit,'(i3,6f10.4)') molecule(j)%nucl_charge,&
         &(molecule(j)%cartcoord(jj)*bohr2ang,jj=1,3),&
         &(spectrum(i)%norm_mode(jj)*scale,jj=j*3-2,j*3)
      end do
    end do
    close(xmol_unit)
    write(out,'(''   Created xmol XYZ input file : '',A)') xmol_file
  end subroutine print_xmol_norm

 !************************************************************************  

end module snf_xmol
