/***************************************************************************
 replacement for system call with enhanced functionality:
 The called task is periodically checked for its state; start of task,
 end of task, uptime record and ps record for the task are communicated
 to the task that called fsystem by means of a call to MNOTIFY
 060801 - IRIX
 190501 - stat
 280101 - SR8000
 BAH, May 16, 1999
 ***************************************************************************/
#define INT int
#ifdef UNDERSCORE
#define MNOTIFY mnotify_
#define TERMPID termpid_
#define FSYSTEM fsystem_
#define ARCHIVING archiving_
#define REQ_CTOK reqctok_
#define REL_CTOK relctok_
#else
#ifdef T3E
#define MNOTIFY MNOTIFY
#define TERMPID TERMPID
#define FSYSTEM FSYSTEM
#define ARCHIVING ARCHIVING
#define REQ_CTOK REQCTOK
#define REL_CTOK RELCTOK
#else
#define MNOTIFY mnotify
#define TERMPID termpid
#define FSYSTEM fsystem
#define ARCHIVING archiving
#define REQ_CTOK reqctok
#define REL_CTOK relctok
#endif
#endif
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>

/* various definitions for different machines        */
/*                                                   */
/* PSCMD, PSPPID   syntax os ps command              */
/* INTERVAL        seconds between machine checks    */
/* SCRFILE         scratch filename                  */
/*                                                   */

#ifdef HIUXWE
#define PSCMD "ps axl"
#define PSPPID "$3"
#define INTERVAL 60
#endif
#ifdef LINUX
#define PSCMD "ps axl"
#define PSPPID "$4"
#define INTERVAL 10
#endif
#ifdef IRIX
#define PSCMD "ps -aef"
#define PSPPID "$3"
#define INTERVAL 60
#endif
#ifdef HP
#define PSCMD "ps -ael"
#define PSPPID "$5"
#define INTERVAL 10
#endif
#if defined(OSF1) || defined(OSFD) || defined(OSFE)
#define PSPPID "$3"
#define PSCMD "ps axl"
#define INTERVAL 10
#endif
#ifdef T3E
#define PSPPID "$5"
#define PSCMD "ps -ael"
#define INTERVAL 10
#endif

#define SCRFILE "/tmp/TMP.XXXXXX.nfk"


/***************************************************************************/
/* global variables                                                        */
/***************************************************************************/
typedef void (*sighandler_t)(INT);           /* typedef for signal handler */
pid_t child_pid;                             /* pid of child task          */
char command[256];                           /* command to be executed     */

/* the following variables are set be the ARCHIVE call                     */

time_t timestamp;                            /* MO file time stamp         */

/* the following variables are set be the ARCHIVING call                   */

char tmpdi[256];                             /* temporary directory        */
char file1[256];                             /* mo filename ('mos','alpha')*/
char file2[256];                             /* mo filename ('beta')       */
char apath[256];                             /* archive path               */
int astepno=0;                               /* archive step number        */
int archno=0;                                /* number of files to archive */
/***************************************************************************/
/* SIGALRM trap                                                            */
/***************************************************************************/
void fsystem_trap(n) INT n; {
int info;
/* printf("fsystem_trap %d child_pid=%d\n",n, child_pid); */
chk_task(child_pid);
#if !defined(T3E) && !defined(HIUXWE)
 info=(INT)signal(SIGALRM,fsystem_trap);
 if (info<0) {
  fprintf(stderr,"fsystem_trap: cannot establish timer trap\n");
  perror("establishing timer trap");
  exit(7);
  }
#endif

alarm(INTERVAL);                             /* set up new alarm           */
}
/***************************************************************************/
/* termpid - terminates child task; called by slshutdown and master        */
/***************************************************************************/
INT TERMPID(pid) INT *pid; {
INT p;
p= *pid;
if (p==0) return(kill(child_pid,SIGTERM));
return(kill(p,SIGTERM));
}
/***************************************************************************/
/* fsystem program                                                         */
/***************************************************************************/
INT FSYSTEM(l,x) char*x; INT *l;
{
 INT one=1,two=2;
 INT info;

 INT pid, status, len;

 len = *l;

 if (len>255) {
  fprintf(stderr,"fsystem: command length (%d) exceeds maximum of 255",
  len);
  exit(9);
  }

 strncpy(command,x,len);
 command[len]='\0';

 pid = fork();
 if (pid == -1)
 return -1;

 /* the child process */
 if (pid == 0) {
  char *argv[4];
  argv[0] = "sh";
  argv[1] = "-c";
  argv[2] = command;
  argv[3] = 0;
  execv("/bin/sh", argv);
  exit(127);
  }

 status=0;
#ifndef T3E
 MNOTIFY(&one,&pid,&status,x,l);             /* notify calling task        */
#endif
 child_pid=pid;                              /* store child pid globally   */

 info=(INT)signal(SIGALRM,fsystem_trap);
 if (info<0) {
  fprintf(stderr,"fsystem: cannot establish timer trap\n");
  perror("establishing timer trap");
  exit(8);
  }

 alarm(INTERVAL);

 do {
  if (waitpid(pid, &status, 0) == -1) {
   if (errno != EINTR) return -1;
   } else
   {
#ifndef T3E
   MNOTIFY(&two,&pid,&status,x,l);           /* notify calling task        */
#endif
   return status;
   }
  } while(1);
 }
/***************************************************************************/
 INT chk_task(ppp) INT ppp;
 {
#if !defined(SMP)
 char cmd[512];
 INT pid,status,len;
 FILE *f;
 char buf[255];
 char filename[255];
 INT three=3, four=4;
 char *p,*q;

 sprintf(filename,"%s%d\0", SCRFILE, ppp);
 mkstemp(filename);

/* ps command and CPU reporting */

 sprintf(cmd,"uptime > %s; %s | awk '{if (%s==\"%d\") print $0}' >> %s\0",
  filename, PSCMD, PSPPID, ppp, filename);
 pid = fork();
 if (pid == -1)
 return -1;

 /* the child process */
 if (pid == 0) {
  char *argv[4];
  argv[0] = "sh";
  argv[1] = "-c";
  argv[2] = cmd;
  argv[3] = 0;
  execv("/bin/sh", argv);
  exit(127);
  }

 status=0;
 do {
  if (waitpid(pid, &status, 0) == -1) {
   if (errno != EINTR) return -1;
   } else
   {

   /* give data back to calling task */
   pid=ppp;
   f=fopen(filename,"r");
   if (!f) {
    fprintf(stderr,"fsystem: cannot open file %s\n",SCRFILE);
    exit(7);
    }

   fgets(buf,255,f);
   p=buf;
   /* load average */
   while (*p && strncmp(p,"age:",4)) p++;
   if (*p) {
    p+=5;
    while (*p && *p==' ') p++;
    q=p; while (*q && *q!=',') q++;
    *q='\0';
    len=strlen(p);
    MNOTIFY(&three,&pid,&status,p,&len);
    }

   /* cpu time */
   fgets(buf,255,f);
   p=buf;
   while (*p && *p!=':') p++;
   if (*p) {
    while (*p!=' ') p--;
    len=strlen(p);
    MNOTIFY(&four,&pid,&status,p,&len);
    }

   fclose(f);
   unlink(filename);
   break;
   }
  } while(1); /* end of wait loop for uptime command */
#endif

  /* if archive step number is different from zero, stat the MO file and
     copy it to archive machine/directory, if the file has been changed */

  if (archno==1) archive(file1,ppp);
  else {
   archive(file1,ppp);
   archive(file2,ppp);
   }
  return 0;
  }
/***************************************************************************/
 INT ARCHIVING(tempdir,moname,arpath,stepno,fileno)
  char *tempdir, *moname, *arpath; INT *stepno, *fileno;
 {
 /* BAH 190501
    enable archiving by setting <astepno> to the current step number
    disable archiving by setting <astepno> to zero
    set the name of the temporary directory, the filename to be archived
    and the archiving machine:directory
    <archno> is either 1 or 2, depending if we have an uhf case or not
    called by FORTRAN subroutine rslave
 */

 if (*stepno <=0) { /* disable archiving altogether */
  astepno=0;
  archno=0;
  return 0;
  }

 astepno= *stepno;

 /* copy MO filename and archive path name */

 forcopy(tmpdi,tempdir); forcopy(apath,arpath);

 if (*fileno != 2) { archno=1; forcopy(file1,moname); }
 else { archno=2; forcopy(file2,moname); }
  
 return 0;
 }
/***************************************************************************/
 INT archive(name,ppp) char *name; int ppp;
 {
 /* BAH 190501
    check the file (mos, alpha, beta) if it was changed since the last call
    of the subroutine; if yes, copy it to the archiving machine/directory
    and notify the master task with icase=6
    called from chk_task
 */

 int status, pid, mypid, len;
 int six=6;
 struct stat statbuf;
 char cmd[512], mofile[512];

 mypid=ppp;

/* debugging */
/* 
 sprintf(cmd,"archive: stepno=%d\n",astepno);
 len=strlen(cmd);
 MNOTIFY(&six,&mypid,&status,cmd,&len);
*/

  if (*apath!='\0' && astepno>0) {
   sprintf(mofile,"%s/%s\0",tmpdi,name);
   status=stat(mofile,&statbuf);
   if (timestamp!=statbuf.st_atime) {

   REQ_CTOK(&mypid);

   timestamp=statbuf.st_atime;

   sprintf(cmd,"scp %s %s/%s.%d\0", mofile, apath, name, astepno);
   pid = fork();
   if (pid == -1) return -1;
   if (pid == 0) {char *argv[4]; argv[0] = "sh";
                                 argv[1] = "-c";
                                 argv[2] = cmd;
                                 argv[3] = 0;
                                 status=execv("/bin/sh", argv);
                                 exit(127);
                                 }

   waitpid(pid,&status,0);
   REL_CTOK(&mypid);
   len=strlen(cmd);
   MNOTIFY(&six,&mypid,&status,cmd,&len);
   }
  }
 return(0);
 }
/***************************************************************************/
 INT forcopy(to,from,maxl) char *to, *from; INT maxl;
 {
 /* BAH 190501
    copy the fortran character <from> as C char to <to>, up to a maximum
    length <maxl>
 */
 INT len;
 len=maxl;
 while (len-- && *from!=' ') {*to++=*from++;}
 *to='\0';
 return 0;
 }
