CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
*****************************************************************************
      SUBROUTINE inictok

* initialize communication token request queue
* called by main program in main.r

      INCLUDE 'defs.h'
      INCLUDE 'strctok.h'

      ctok_first=0
      ctok_last=0
      ctok_n=0

* maximum number of parallel communication requests

      ctok_max=4

      RETURN
      END
*****************************************************************************
      SUBROUTINE setmax_ctok(m)

* set maximal number of communication channels

      INCLUDE 'defs.h'
      INCLUDE 'strctok.h'
      INTEGER m
      ctok_max=m
      RETURN
      END
*****************************************************************************
      SUBROUTINE enq_ctok(tid,pid)

* enqueue entry into communication token request queue
* called by areqctok

      INCLUDE 'defs.h'
      INCLUDE 'strctok.h'
      INCLUDE 'objects.h'
      INTEGER pid,tid

      INTEGER p
      CALL memmgr(memget,'ctok    ',is,p,strctok_len)
      ctok_n=ctok_n+1
      is(p+strctok_next)=0
      is(p+strctok_tid)=tid
      is(p+strctok_pid)=tid
      IF (ctok_first.EQ.0) THEN
       ctok_first=p
       ctok_last=p
       RETURN
      ENDIF
      is(ctok_last+strctok_next)=p
      ctok_last=p
      RETURN
      END
*****************************************************************************
      SUBROUTINE deq_ctok(tid,pid)

* get first entry from communication token queue
* called by grant_ctok

      INCLUDE 'defs.h'
      INCLUDE 'strctok.h'
      INCLUDE 'objects.h'
      INTEGER pid,tid,p,q

      tid=0
      pid=0
      IF (ctok_first.EQ.0) RETURN
      p=ctok_first
      tid=is(p+strctok_tid)
      pid=is(p+strctok_pid)
      q=is(p+strctok_next)
      ctok_first=q
      IF (p.EQ.0) ctok_last=0
      ctok_n=ctok_n-1
      CALL memmgr(memfre,'ctok    ',is,p,strctok_len)
      RETURN
      END
*****************************************************************************
      SUBROUTINE reqctok(pid)
*
* request a communication token from the master
* called by the slave process to obtain permission to communicate with
* the archiving machine
*
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      INCLUDE 'param.h'
      INTEGER pid
      INTEGER w
C#    INTEGER st(MPI_STATUS_SIZE)
C#    CHARACTER*(MPI_MAX_PROCESSOR_NAME) mpiname
      INTEGER token,irc,info
      WRITE (iosl,*) 'reqctok ',pid
      CALL flush(iosl)
      irc=0
      CALL mnotify(7,pid,irc,' ',0)
C#    w=MPI_COMM_WORLD
C#    CALL MPI_RECV(token,1,MPI_INTEGER,tid0,419,w,st,info)
C#    CALL mpierr('MPIRECV pgetstr','reqtok',irc)
C//SKIPC#
      CALL flush(iosl)
      CALL PVMFRECV(tid0,0,w)
      CALL PVMFUNPACK(INTEGER4,token,1,1,info)
      IF (irc.NE.0) CALL PVMFPERROR('PVMRECV reqctok',irc)
C//END
      WRITE (iosl,*) ' token received: ',token
      CALL flush(iosl)
      RETURN
      END
*****************************************************************************
      SUBROUTINE relctok(pid)
*
* release communication token
* called by slave process to give back a communication token to the master
*
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      INTEGER pid,irc
      irc=0
      CALL mnotify(8,pid,irc,' ',0)
      WRITE (iosl,*) 'relctok token released, pid=',pid
      CALL flush(iosl)
      RETURN
      END
*****************************************************************************
      SUBROUTINE areqctok(tid4,pid4)
*
* acknowledge request of communication token
*
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      INCLUDE 'strctok.h'
      INTEGER*4 tid4,pid4
      INTEGER p
      WRITE (idbg,*) 'areqctok ',tid4,pid4,' waiting: ',ctok_n
*
*     enqueue into communication token request queue      

      CALL enq_ctok(tid4,pid4)

* debug
C@    WRITE (*,*) '--- ctok queue ---'
C@    p=ctok_first
C@1   IF (p.EQ.0) GOTO 2
C@    WRITE (*,*) p,is(p+strctok_next),is(p+strctok_tid),
C@   * is(p+strctok_pid)
C@    p=is(p+strctok_next)
C@    GOTO 1
C@2   CONTINUE

      RETURN
      END
*****************************************************************************
      SUBROUTINE grant_ctok
*
* grant communication tokens
*
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      INCLUDE 'strctok.h'
      INTEGER*4 tid,pid
C#    INTEGER w
C#    INTEGER st(MPI_STATUS_SIZE)
C#    CHARACTER*(MPI_MAX_PROCESSOR_NAME) mpiname
      INTEGER token,irc,info,mbuf

1     IF (ctok_max.LE.0) RETURN
      CALL deq_ctok(tid,pid)
      IF (tid.EQ.0) RETURN
C@    WRITE (*,*) '@grant_ctok@ tid ',tid
C#    w=MPI_COMM_WORLD
C:    CALL MPI_SSEND(ctok_max,1,MPI_INTEGER,tid,419,w,info)
C:    CALL mpierr('MPI_SSEND','grant_ctok',info)
C//SKIPC:2
C#    CALL MPI_SEND(ctok_max,1,MPI_INTEGER,tid,419,w,info)
C#    CALL mpierr('MPI_SEND','grant_ctok',info)
C//SKIPC#
      CALL PVMFINITSEND(PVMDATADEFAULT,mbuf)
      CALL PVMFPACK(INTEGER4,ctok_max,1,1,info)
      CALL PVMFSEND(tid,0,info)
C//END
      WRITE (idbg,*) 'grantctok ',tid,pid,ctok_max
      ctok_max=ctok_max-1
      GOTO 1
      END
*****************************************************************************
      SUBROUTINE arelctok(tid4,pid4)
*
* acknowledge release of communication token
*
      INCLUDE 'defs.h'
      INCLUDE 'comm.h'
      INCLUDE 'strctok.h'
      INTEGER*4 tid4,pid4
      WRITE (idbg,*) 'arelctok ',tid4,pid4
      ctok_max=ctok_max+1
      RETURN
      END
